# Copyright 2013 Cisco Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module contains APIs to facilitate UcsCentral backup and import
"""

import os
import platform
import time
import datetime
import logging
from ..ucscexception import UcscValidationException, \
    UcscWarning, \
    UcscOperationError

log = logging.getLogger('ucscentral')


ucsc_base_dn = "org-root/deviceprofile-default"


def _validate_remote_host_args(protocol, hostname, username, password):
    if not protocol:
        raise UcscValidationException("Missing protocol argument")
    if not hostname:
        raise UcscValidationException("Missing hostname argument")
    if protocol == 'tftp':
        return

    if not username:
        raise UcscValidationException("Missing username argument")
    if not password:
        raise UcscValidationException("Missing password argument")


def _backup(handle, file_dir, file_name, timeout=600,
            remote_enabled=False, protocol=None,
            hostname="localhost", username=None, password="",
            remove_from_ucsc=False,
            preserve_pooled_values=False):
    """
    _backup internal method helps create UcsCentral full-state backup and
    download it locally or to remote location.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        file_dir (str): directory to download backup file to
        file_name (str): name for the backup file
        timeout (number) : time in seconds for which method waits
                           for the backUp file to generate before it exits.
        remote_enabled (boolean): True if Remote backup is enabled
                                  False - by default
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        hostname (str): Hostname/IP for the remote backup
        username (str): Username for remote backup
        password (str): Password for remote backup
        remove_from_ucsc (boolean): True/False, False - by default
        preserve_pooled_values (boolean): True/False, False - by default

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "config_backup.tgz"\n
        _backup(handle, file_dir=file_dir, file_name=file_name)\n

        _backup(handle, file_dir=file_dir, file_name=file_name,
                remote_enabled=True, protocol='scp',hostname='192.168.1.1',
                username='admin',password='password')\n

    """
    from ..mometa.mgmt.MgmtBackup import MgmtBackup, MgmtBackupConsts
    from ..mometa.top.TopSystem import TopSystem

    backup_type = "full-state"

    if not file_dir:
        raise UcscValidationException("Missing file_dir argument")
    if not file_name:
        raise UcscValidationException("Missing file_name argument")

    top_system = TopSystem()

    if remote_enabled:
        _validate_remote_host_args(protocol, hostname, username, password)
        if (not file_name.endswith('.tgz')):
            raise UcscValidationException(
                "file_name must be .tgz format")

        file_path = os.path.join(file_dir, file_name)
        mgmt_backup = MgmtBackup(
                parent_mo_or_dn=top_system,
                hostname=hostname,
                admin_state=MgmtBackupConsts.ADMIN_STATE_ENABLED,
                proto=protocol,
                type=backup_type,
                remote_file=file_path,
                user=username,
                pwd=password)
    else:
        if not os.path.exists(file_dir):
            os.makedirs(file_dir)

        file_string = platform.node().lower() \
            + datetime.datetime.now().strftime("%Y%m%d%H%M%S")
        file_path = "/" + file_string + "_" + backup_type + "_backup.tgz"

        mgmt_backup = MgmtBackup(
                parent_mo_or_dn=top_system,
                hostname=hostname,
                admin_state=MgmtBackupConsts.ADMIN_STATE_ENABLED,
                proto=MgmtBackupConsts.PROTO_HTTP,
                type=backup_type,
                remote_file=file_path)

    if preserve_pooled_values:
        mgmt_backup.preserve_pooled_values = \
            MgmtBackupConsts.PRESERVE_POOLED_VALUES_YES
    else:
        mgmt_backup.preserve_pooled_values = \
            MgmtBackupConsts.PRESERVE_POOLED_VALUES_NO

    handle.add_mo(mgmt_backup, modify_present=True)
    handle.commit()

    mgmt_backup = handle.query_dn(dn=mgmt_backup.dn)
    admin_state_temp = mgmt_backup.admin_state
    # Checking for the backup to compete.
    duration = timeout
    poll_interval = 2

    log.debug("Starting Backup ")
    while True:
        mgmt_backup = handle.query_dn(dn=mgmt_backup.dn)
        admin_state_temp = mgmt_backup.admin_state

        # Break condition:- if state id disabled then break
        if admin_state_temp == MgmtBackupConsts.ADMIN_STATE_DISABLED:
            break

        time.sleep(min(duration, poll_interval))
        duration = max(0, (duration - poll_interval))
        if duration == 0:
            handle.remove_mo(mgmt_backup)
            handle.commit()
            raise UcscOperationError(
                  "Backup UcsCentral", " operation timed out")

    if remote_enabled:
        if mgmt_backup.over_all_status == \
                MgmtBackupConsts.OVER_ALL_STATUS_FAILED:
            log.debug("Backup failed")
            handle.remove_mo(mgmt_backup)
            handle.commit()
            raise UcscOperationError(
                "Backup UcsCentral", "%s" % mgmt_backup.fsm_rmt_inv_err_descr)

    if not remote_enabled:
        file_source = "backupfile" + file_path
        if handle.is_local_download_supported():
            try:
                log.debug("Starting Download ")
                handle.file_download(url_suffix=file_source,
                                     file_dir=file_dir,
                                     file_name=file_name)
            except Exception as err:
                log.debug("Download backup Failed")
                UcscWarning(str(err))
        else:
            log.error("Local download not supported from sdk for this "
                      "version of UcsCentral, skipping it")

    if remove_from_ucsc:
        # remove backup from UcsCentral
        log.debug("Removing backup from UcsCentral")
        handle.remove_mo(mgmt_backup)
        handle.commit()


def backup_local(handle, file_dir, file_name, preserve_pooled_values=False,
                 remove_from_ucsc=False, timeout=600):
    """
    backup_local helps create UcsCentral full-state backup and dowload it
    locally.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        file_dir (str): directory to download ucs backup file to
        file_name (str): name for the backup file
        preserve_pooled_values (boolean): True/False, False - by default
        remove_from_ucsc (boolean): True/False, False - by default
        timeout (number) : time in seconds for which method waits
                           for the backUp file to generate before it exits.

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "full-state_backup.tgz"\n
        backup_local(handle, file_dir=file_dir, file_name=file_name)\n
    """
    _backup(handle, file_dir=file_dir, file_name=file_name,
            remote_enabled=False, hostname="localhost",
            preserve_pooled_values=preserve_pooled_values,
            remove_from_ucsc=remove_from_ucsc,
            timeout=timeout)


def backup_remote(handle, file_dir, file_name, hostname,
                  protocol="scp", username=None, password="",
                  preserve_pooled_values=False,
                  remove_from_ucsc=False,
                  timeout=600):
    """
    backup_remote helps create and download UcsCentral full-state backup to
    remote location.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        file_dir (str): directory to download ucs backup file to
        file_name (str): name for the backup file
                         (supported file extension '.tgz')
        hostname (str): Hostname/IP for the remote backup
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        username (str): Username for remote backup
        password (str): Password for remote backup
        preserve_pooled_values (boolean): True/False, False - by default
        remove_from_ucsc (boolean): True/False, False - by default
        timeout (number) : time in seconds for which method waits
                           for the backUp file to generate before it exits.

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "full-state_backup.tgz"\n
        backup_remote(handle, file_dir=file_dir, file_name=file_name,
                      protocol='scp',hostname='192.168.1.1',
                      username='admin',password='password')\n

    """
    _backup(handle, file_dir=file_dir, file_name=file_name,
            remote_enabled=True,
            hostname=hostname, protocol=protocol,
            username=username, password=password,
            preserve_pooled_values=preserve_pooled_values,
            remove_from_ucsc=remove_from_ucsc,
            timeout=timeout)


def _export_config(handle, file_dir, file_name, timeout=600,
                   remote_enabled=False, protocol=None,
                   hostname="localhost", username=None, password="",
                   remove_from_ucsc=False,
                   preserve_pooled_values=False):
    """
    _export_config internal method helps export UcsCentral config-all backup
    and download it locally or to remote location.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        file_dir (str): directory to download ucs backup file to
        file_name (str): name for the backup file
        timeout (number) : time in seconds for which method waits
                              for the backUp file to generate before it exits.
        remote_enabled (boolean): True if Remote backup is enabled
                                  False - by default
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        hostname (str): Hostname/IP for the remote backup
        username (str): Username for remote backup
        password (str): Password for remote backup
        remove_from_ucsc (boolean): True/False, False - by default
        preserve_pooled_values (boolean): True/False, False - by default

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "export_config.tgz"\n
        _export_config(handle, file_dir=test_support,
                       file_name=file_name)\n

        _export_config(handle, file_dir=test_support,
                    file_name=file_name,remote_enabled=True,
                    protocol='scp',hostname='192.168.1.1',
                    username='admin',password='password')\n

    """
    from ..mometa.mgmt.MgmtDataExporter import MgmtDataExporter, \
        MgmtDataExporterConsts
    from ..mometa.top.TopSystem import TopSystem

    backup_type = "config-all"
    if not file_dir:
        raise UcscValidationException("Missing file_dir argument")
    if not file_name:
        raise UcscValidationException("Missing file_name argument")

    top_system = TopSystem()

    if remote_enabled:
        _validate_remote_host_args(protocol, hostname, username, password)
        if (not file_name.endswith('.tgz')):
            raise UcscValidationException(
                "file_name must be .tgz format")

        file_path = os.path.join(file_dir, file_name)
        mgmt_export = MgmtDataExporter(
                parent_mo_or_dn=top_system,
                hostname=hostname,
                admin_state=MgmtDataExporterConsts.ADMIN_STATE_ENABLED,
                proto=protocol,
                type=backup_type,
                remote_file=file_path,
                user=username,
                pwd=password)
    else:
        if not os.path.exists(file_dir):
            os.makedirs(file_dir)

        file_string = platform.node().lower() \
            + datetime.datetime.now().strftime("%Y%m%d%H%M%S")
        file_path = "/" + file_string + "_" + backup_type + "_backup.tgz"

        mgmt_export = MgmtDataExporter(
                parent_mo_or_dn=top_system,
                hostname=hostname,
                admin_state=MgmtDataExporterConsts.ADMIN_STATE_ENABLED,
                proto=MgmtDataExporterConsts.PROTO_HTTP,
                type=backup_type,
                remote_file=file_path)

    if preserve_pooled_values:
        mgmt_export.preserve_pooled_values = \
            MgmtDataExporterConsts.PRESERVE_POOLED_VALUES_YES
    else:
        mgmt_export.preserve_pooled_values = \
            MgmtDataExporterConsts.PRESERVE_POOLED_VALUES_NO

    handle.add_mo(mgmt_export, modify_present=True)
    handle.commit()

    mgmt_export = handle.query_dn(dn=mgmt_export.dn)
    admin_state_temp = mgmt_export.admin_state
    log.debug("Starting export config")
    # Checking for the backup to compete.
    duration = timeout
    poll_interval = 2

    while True:
        mgmt_export = handle.query_dn(dn=mgmt_export.dn)
        admin_state_temp = mgmt_export.admin_state

        # Break condition:- if state id disabled then break
        if admin_state_temp == MgmtDataExporterConsts.ADMIN_STATE_DISABLED:
            break

        time.sleep(min(duration, poll_interval))
        duration = max(0, (duration - poll_interval))
        if duration == 0:
            handle.remove_mo(mgmt_export)
            handle.commit()
            raise UcscOperationError(
                "Config Export UcsCentral", "operation timed out")

    if remote_enabled:
        if mgmt_export.over_all_status == \
                MgmtDataExporterConsts.OVER_ALL_STATUS_FAILED:
            log.debug("Config exported failed")
            handle.remove_mo(mgmt_export)
            handle.commit()
            raise UcscOperationError(
                "Config Export UcsCentral", "%s" % mgmt_export.fsm_rmt_inv_err_descr)

    if not remote_enabled:
        file_source = "backupfile" + file_path
        if handle.is_local_download_supported():
            try:
                log.debug("Starting download")
                handle.file_download(url_suffix=file_source,
                                     file_dir=file_dir,
                                     file_name=file_name)
            except Exception as err:
                log.debug("Download export config Failed")
                UcscWarning(str(err))
        else:
            log.error("Local download not supported from sdk for this "
                      "version of UcsCentral, skipping it")

    if remove_from_ucsc:
        # remove backup from UcsCentral
        log.debug("Removing export config from UcsCentral")
        handle.remove_mo(mgmt_export)
        handle.commit()


def export_config_local(handle, file_dir, file_name,
                        preserve_pooled_values=False,
                        remove_from_ucsc=False,
                        timeout=600):
    """
    export_config_local helps export UcsCentral config-all backup and download
    it locally.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        file_dir (str): directory to download ucs backup file to
        file_name (str): name for the backup file
        preserve_pooled_values (boolean): True/False, False - by default
        remove_from_ucsc (boolean): True/False, False - by default
        timeout (number) : time in seconds for which method waits
                           for the backUp file to generate before it exits.

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "export_config.tgz"\n
        export_config_local(handle, file_dir=file_dir,
                            file_name=file_name)\n
    """

    _export_config(handle, file_dir=file_dir, file_name=file_name,
                   remote_enabled=False, hostname="localhost",
                   preserve_pooled_values=preserve_pooled_values,
                   remove_from_ucsc=remove_from_ucsc,
                   timeout=timeout)


def export_config_remote(handle, file_dir, file_name, hostname,
                         protocol="scp", username=None, password="",
                         preserve_pooled_values=False,
                         remove_from_ucsc=False,
                         timeout=600):
    """
    export_config_remote helps export UcsCentral config-all backup and
    download it to remote location.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        file_dir (str): directory to download ucs backup file to
        file_name (str): name for the backup file
                         (supported file extension '.tgz')
        hostname (str): Hostname/IP for the remote backup
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        username (str): Username for remote backup
        password (str): Password for remote backup
        preserve_pooled_values (boolean): True/False, False - by default
        remove_from_ucsc (boolean): True/False, False - by default
        timeout (number) : time in seconds for which method waits
                           for the backUp file to generate before it exits.

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "export_config.tgz"\n

        export_config_remote(handle, file_dir=file_dir,
                    file_name=file_name,
                    protocol='scp',hostname='192.168.1.1',
                    username='admin',password='password')\n

    """
    _export_config(handle, file_dir=file_dir, file_name=file_name,
                   remote_enabled=True,
                   hostname=hostname, protocol=protocol,
                   username=username, password=password,
                   preserve_pooled_values=preserve_pooled_values,
                   remove_from_ucsc=remove_from_ucsc,
                   timeout=timeout)


def _fail_and_remove_domain_backup(handle, backup_status_mo, err):
    if backup_status_mo:
        handle.remove_mo(backup_status_mo)
        handle.commit(dme="resource-mgr")
    raise UcscOperationError("Domain backup/export_config", err)


def _backup_or_exportconfig_domain(handle, backup_type, file_dir, file_name,
                                   domain_ip, domain_name, hostname,
                                   preserve_pooled_values, protocol,
                                   username, password, timeout):
    """
    This internal function helps create domain full_state backup or export
    config to remote location
    Note: This is internal function, should use either backup_domain or
    backup_export_config
    Args:
        handle (UcscHandle): UcsCentral Connection handle
        file_dir (str): directory to download ucs backup file to
        file_name (str): name for the backup file
                         (supported file extension is '.tgz')
        backup_type (str): Either full-state or config-all.
        timeout (number) : time in seconds for which method waits
                           for the backUp file to generate before it exits.
        domain_ip(str): IP of domain, set 'None' if domain_name is valid
        domain_name(str): Domain name, valid only if domain_ip is None
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        hostname (str): Hostname/IP for the remote backup
        username (str): Username for remote backup
        password (str): Password for remote backup
        preserve_pooled_values (boolean): True/False,
                                          False - by default

    """
    from ..mometa.mgmt.MgmtBackupOperation import MgmtBackupOperation, \
        MgmtBackupOperationConsts
    from ..mometa.mgmt.MgmtBackup import MgmtBackupConsts
    from .ucscdomain import get_domain, _is_domain_available

    preserve_pooled_values = False

    if not file_dir:
        raise UcscValidationException("Missing file_dir argument")
    if not file_name:
        raise UcscValidationException("Missing file_name argument")
    if not domain_ip:
        raise UcscValidationException("Missing domain_ip argument")

    if backup_type == 'full-state':
        if (not file_name.endswith('.tgz')):
            raise UcscValidationException(
                "file_name must be .tgz format")
    elif backup_type == 'config-all':
        if (not file_name.endswith('.xml')):
            raise UcscValidationException(
                "file_name must be .xml format")
    _validate_remote_host_args(protocol, hostname, username, password)

    domain = get_domain(handle, domain_ip, domain_name)

    if _is_domain_available(handle, domain.id):
        domain_dn = domain.dn
    else:
        raise UcscOperationError("Backup or Export_config",
                                 "Domain with IP %s or name %s not "
                                 "registered or lost visibility "
                                 "with UcsCentral" %
                                 (domain_ip, domain_name))

    file_path = os.path.join(file_dir, file_name)

    mgmt_backup = MgmtBackupOperation(
            parent_mo_or_dn=domain_dn,
            hostname=hostname,
            admin_state=MgmtBackupOperationConsts.ADMIN_STATE_ENABLED,
            proto=protocol,
            type=backup_type,
            remote_file=file_path,
            user=username,
            pwd=password)
    if preserve_pooled_values:
        mgmt_backup.preserve_pooled_values = \
            MgmtBackupOperationConsts.PRESERVE_POOLED_VALUES_YES
    else:
        mgmt_backup.preserve_pooled_values = \
            MgmtBackupOperationConsts.PRESERVE_POOLED_VALUES_NO

    handle.set_mo(mgmt_backup)
    handle.commit()

    log.debug("Triggering Domain Backup ")
    duration = 30
    poll_interval = 2
    backup_status_dn = "extpol/reg/clients/client-" + \
        domain.id + "/backup-" + hostname
    while True:
        backup_status = handle.query_dn(
            dn=backup_status_dn, dme="resource-mgr")
        if backup_status is not None:
            break

        time.sleep(min(duration, poll_interval))
        duration = max(0, (duration - poll_interval))
        if duration == 0:
            raise UcscOperationError(
                "Backup or export config of domain", "not triggered")

    log.debug("Domain Backup Triggered")

    # Checking for the backup to become available.
    log.debug("Waiting for Domain Backup to become available")
    duration = timeout
    poll_interval = 5

    while True:
        backup_status = handle.query_dn(backup_status_dn, dme="resource-mgr")
        if backup_status.over_all_status == \
                MgmtBackupConsts.OVER_ALL_STATUS_ALL_SUCCESS:
            break
        if backup_status.over_all_status != \
                MgmtBackupConsts.OVER_ALL_STATUS_WORK_IN_PROGRESS:
            _fail_and_remove_domain_backup(
                handle, backup_status, 'operation failed')
        time.sleep(min(duration, poll_interval))
        duration = max(0, (duration - poll_interval))
        if duration == 0:
            _fail_and_remove_domain_backup(
                handle, backup_status, 'operation timed out')

    log.debug("Domain backup is available")


def backup_domain_remote(handle, file_dir, file_name,
                         domain_ip,  protocol, hostname,
                         username=None, password="",
                         domain_name=None, preserve_pooled_values=False,
                         timeout=600):
    """
    backup_domain_remote  helps create domain full_state backup and download
    it to remote location.
    Note: Domain backup will always be remote backup
    Args:
        handle (UcscHandle): UcsCentral Connection handle
        file_dir (str): directory to download ucs backup file to
        file_name (str): name for the backup file
                         (supported file extension is '.tgz')
        timeout (number) : time in seconds for which method waits
                              for the backUp file to generate before it exits.
        domain_ip(str): IP of domain, set 'None' if domain_name is valid
        domain_name(str): Domain name, valid only if domain_ip is None
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        hostname (str): Hostname/IP for the remote backup
        username (str): Username for remote backup
        password (str): Password for remote backup
        preserve_pooled_values (boolean): True/False,
                                          False - by default

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "domain_backup.tgz"\n

        backup_domain_remote(handle, file_dir=file_dir, file_name=file_name,
                    domain_ip='10.10.10.1', protocol='scp',
                    hostname='192.168.1.1',
                    username='admin',password='password')\n

    """
    backup_type = "full-state"
    return _backup_or_exportconfig_domain(handle, backup_type, file_dir,
                                          file_name, domain_ip, domain_name,
                                          hostname, preserve_pooled_values,
                                          protocol, username, password,
                                          timeout)


def export_config_domain_remote(handle, file_dir, file_name,
                                domain_ip, hostname,  protocol,
                                username=None, password="",
                                domain_name=None, preserve_pooled_values=False,
                                timeout=600):
    """
    export_config_domain_remote helps create domain export config and download
    it to remote location.
    Note: Domain export export config will always be remote export

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        file_dir (str): directory to download ucs backup file to
        file_name (str): name for the backup file
                         (supported file extension is '.xml')
        timeout (number) : time in seconds for which method waits
                           for the backUp file to generate before it exits.
        domain_ip(str): IP of domain, set 'None' if domain_name is valid
        domain_name(str): Domain name, valid only if domain_ip is None
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        hostname (str): Hostname/IP for the remote backup
        username (str): Username for remote backup
        password (str): Password for remote backup
        preserve_pooled_values (boolean): True/False,
                                          False - by default

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "config_backup.xml"\n
        export_config_domain(handle, file_dir=test_support,
                    file_name=file_name,
                    username='admin', password='password',
                    domain_ip='10.10.10.1', protocol='scp',
                    hostname='192.168.1.1')
    """
    backup_type = "config-all"
    return _backup_or_exportconfig_domain(handle, backup_type, file_dir,
                                          file_name, domain_ip, domain_name,
                                          hostname, preserve_pooled_values,
                                          protocol, username, password,
                                          timeout)


def _is_backup_file_on_server(handle, hostname_str, backup_file):

    backup_path_str = "/" + hostname_str + "/cfg-backups"
    filter_str = '(file_path, %s, type="eq")' % backup_path_str

    backups = handle.query_classid(
        class_id='ConfigBackup', filter_str=filter_str)

    for backup in backups:
        if backup.file_name == backup_file:
            log.debug("Backup file '%s' exist on UcsCentral" % backup_file)
            return True
    return False


def _import_config(handle, file_name, file_location="ucscentral",
                   file_dir=None, merge=True, protocol=None,
                   hostname="localhost",
                   username=None, password="", timeout=120):
    """
    This internal method imports a UcsCentral config from local, remote or
    ucscentral

    Args:
        handle (UcscHandle): connection handle
        file_name (str): backup file name to be imported
        file_location (str): file location where the config file is, it can be:
                             ['ucscentral','local','remote']
        file_dir (str): directory containing ucscentral backup file,
                        used only for import from local or remote
        merge (boolean): True/False, specifies whether to merge the backup
                        config with the existing UCS Central configuration

        protocol (str): Transfer protocol for remote import
                        ['ftp','sftp','tftp','scp']
        hostname (str): Hostname/IP for the remote import
        username (str): Username for remote import
        password (str): Password for remote import
        timeout (number) : time in seconds for which method waits for
                           import operation success before timing out

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "config_export.tgz"\n
        import from ucscentral:
        _import_config(handle, file_name=file_name,
                       file_location="ucscentral")

        import from local:
        _import_config(handle, file_name=file_name,
                       file_location="local",
                       file_dir=file_dir,
                       merge=True)\n

        import from remote:
        _import_config(handle, file_name=file_name,
                       file_location="remote",
                       file_dir=file_dir
                       protocol='scp',hostname='192.168.1.1',
                       username='admin',password='password')\n

    """

    from ..mometa.top.TopSystem import TopSystem
    from ..mometa.mgmt.MgmtDataImporter import MgmtDataImporter, \
        MgmtDataImporterConsts

    if not file_name:
        raise UcscValidationException("Missing file_name argument")

    if file_location != "ucscentral":
        if not file_dir:
            raise UcscValidationException("Missing file_dir argument")

    if (not file_name.endswith('.tgz')):
        raise UcscValidationException("file_name must be .tgz format")

    top_system = TopSystem()

    if file_location == "remote":
        file_path = os.path.join(file_dir, file_name)
        _validate_remote_host_args(protocol, hostname, username, password)
        mgmt_importer = MgmtDataImporter(
            parent_mo_or_dn=top_system,
            hostname=hostname,
            remote_file=file_path,
            proto=protocol,
            user=username,
            pwd=password,
            admin_state=MgmtDataImporterConsts.ADMIN_STATE_ENABLED)

    elif file_location == "local":
        file_path = os.path.join(file_dir, file_name)
        if not os.path.exists(file_path):
            raise UcscOperationError("Import config",
                                     "Backup File '%s' not found" %
                                     file_path)
        mgmt_importer = MgmtDataImporter(
            parent_mo_or_dn=top_system,
            hostname="localhost",
            remote_file='/' + file_name,
            proto=MgmtDataImporterConsts.PROTO_HTTP,
            admin_state=MgmtDataImporterConsts.ADMIN_STATE_ENABLED)

    elif file_location == "ucscentral":
        if not _is_backup_file_on_server(handle, "ucs-central", file_name):
            raise UcscOperationError("Import config",
                                     "Backup File '%s' not found "
                                     "on UcsCentral" % file_name)
        mgmt_importer = MgmtDataImporter(
            parent_mo_or_dn=top_system,
            hostname="localhost",
            remote_file='/ucs-central/cfg-backups/' + file_name,
            proto=MgmtDataImporterConsts.PROTO_TFTP,
            admin_state=MgmtDataImporterConsts.ADMIN_STATE_ENABLED)

    else:
        raise UcscOperationError(
                "Import config",
                "Invalid file_location argument."
                "It must be either ucscentral,local or remote")

    if merge:
        mgmt_importer.action = MgmtDataImporterConsts.ACTION_MERGE
    else:
        mgmt_importer.action = MgmtDataImporterConsts.ACTION_REPLACE

    if file_location == "local":
        try:
            log.debug("Start uploading config")
            uri_suffix = "operations/file-%s/importconfig.txt?Cookie=%s" % (
                         file_name, handle.cookie)
            handle.file_upload(url_suffix=uri_suffix,
                               file_dir=file_dir,
                               file_name=file_name)

        except Exception as err:
            UcscWarning(str(err))
            raise UcscOperationError("Upload config", "upload failed")

    handle.add_mo(mgmt_importer, modify_present=True)
    handle.commit()

    duration = timeout
    poll_interval = 2
    log.debug("Importing UcsCentral config")
    while True:
        mgmt_importer = handle.query_dn(dn=mgmt_importer.dn)
        admin_state = mgmt_importer.admin_state

        # Break condition:- if state id disabled then break
        if admin_state == MgmtDataImporterConsts.ADMIN_STATE_DISABLED:
            break

        time.sleep(min(duration, poll_interval))
        duration = max(0, (duration - poll_interval))
        if duration == 0:
            raise UcscOperationError(
                  "Import config", "operation timed out")

    if mgmt_importer.over_all_status != \
            MgmtDataImporterConsts.OVER_ALL_STATUS_ALL_SUCCESS:
            raise UcscOperationError(
                  "Import config",
                  ("operational status %s " % mgmt_importer.over_all_status))

    log.debug("Import config to UcsCentral was successfull")
    return mgmt_importer


def import_config_ucscentral(handle, file_name, merge=True,
                             timeout=120):
    """
    import_config_ucscentral imports existing UcsCentral config ie.available
    on UcsCentral(taken via schedule_export_config) to the UcsCentral

    Args:
        handle (UcscHandle): connection handle
        file_name (str): backup file name to be imported
        merge (boolean): True/False, specifies whether to merge the backup
                        config with the existing UCS Central configuration
        timeout (number) : time in seconds for which method waits for
                           import operation success before timing out

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "config_export.tgz"\n
        import from ucscentral:
        import_config_ucscentral(handle, file_name=file_name)
    """
    _import_config(handle, file_name=file_name, merge=merge,
                   file_location="ucscentral",
                   hostname="localhost",
                   timeout=timeout)


def import_config_local(handle, file_dir, file_name, merge=True,
                        timeout=120):
    """
    import_config_ucscentral imports UcsCentral config available locally to
    UcsCentral

    Args:
        handle (UcscHandle): connection handle
        file_dir (str): directory containing ucscentral backup file,
        file_name (str): backup file name to be imported
        merge (boolean): True/False, specifies whether to merge the backup
                        config with the existing UCS Central configuration
        timeout (number) : time in seconds for which method waits for
                           import operation success before timing out

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "config_export.tgz"\n

        import from local:
        import_config_local(handle, file_name=file_name,
                            file_dir=file_dir,
                            merge=True)\n

    """
    _import_config(handle, file_name=file_name, file_dir=file_dir, merge=merge,
                   file_location="local",
                   hostname="localhost",
                   timeout=timeout)


def import_config_remote(handle, file_dir, file_name, hostname,
                         merge=True,
                         protocol="scp",
                         username=None, password="",
                         timeout=120):
    """
    import_config_remote imports UcsCentral config from remote location to
    UcsCentral

    Args:
        handle (UcscHandle): connection handle
        file_dir (str): directory containing ucscentral backup file,
        file_name (str): backup file name to be imported
        hostname (str): Hostname/IP for the remote import
        merge (boolean): True/False, specifies whether to merge the backup
                        config with the existing UCS Central configuration
        protocol (str): Transfer protocol for remote import
                        ['ftp','sftp','tftp','scp']
        username (str): Username for remote import
        password (str): Password for remote import
        timeout (number) : time in seconds for which method waits for
                           import operation success before timing out

    Example:
        file_dir = "/home/user/backup"\n
        file_name = "config_export.tgz"\n

        import from remote:
        import_config_remote(handle, file_name=file_name,
                             file_dir=file_dir
                             protocol='scp',hostname='192.168.1.1',
                             username='admin',password='password')\n

    """
    _import_config(handle, file_name=file_name, file_dir=file_dir, merge=merge,
                   file_location="remote",
                   protocol=protocol, hostname=hostname,
                   username=username, password=password,
                   timeout=timeout)


def import_config_domain(handle, to_domain_ip, from_domain_ip, config_file,
                         merge=True,
                         to_domain_name=None, from_domain_name=None,
                         timeout=120):
    """
    This operation imports UcsCentral Domain's config created earlier via
    schedule backup of the same or other Domains.
    Note: Only config-all backup on UcsCentral are available for import, local
    or remote import is not supported for domain

    Args:
        handle (UcscHandle): connection handle
        to_domain_ip(str): IP of domain To which you want to import
                           set 'None' if domain_name is valid
        from_domain_ip(str): IP of domain From which you want to import
                             set 'None' if domain_name is valid
        config_file(str): From domain's config file which you want to import
        merge (boolean): True/False, specifies whether to merge the backup
                         config with the existing domain configuration
        to_domain_name(str): to domain name, valid only if to_domain_ip is None
        from_domain_name(str): from domain name, valid only if
                               from_domain_ip is None
        timeout (number) : time in seconds for which method waits for
                           import operation success before timing out

    Example:
        import_config_domain(handle, to_domain_ip="10.10.10.100",
                             from_domain_ip="192.168.1.1",
                             config_file="all-cfg.1.tgz")\n

    """

    from ..mometa.mgmt.MgmtImporter import MgmtImporter, MgmtImporterConsts
    from .ucscdomain import get_domain, _is_domain_available

    to_domain = get_domain(handle, to_domain_ip, to_domain_name)

    if not _is_domain_available(handle, to_domain.id):
        raise UcscOperationError("Import config domain",
                                 "Domain with IP %s or name %s not "
                                 "registered or lost visibility "
                                 "with UcsCentral" %
                                 (to_domain_ip, to_domain_name))

    from_domain = get_domain(handle, from_domain_ip, from_domain_name)

    if not _is_domain_available(handle, from_domain.id):
        raise UcscOperationError("Import config domain",
                                 "Domain with IP %s or name %s not "
                                 "registered or lost visibility "
                                 "with UcsCentral" %
                                 (from_domain_ip, from_domain_name))

    if not _is_backup_file_on_server(handle, from_domain.address, config_file):
        raise UcscOperationError("Import config domain",
                                 "Backup File '%s' not found "
                                 "on UcsCentral" % config_file)

    filter_str = '(connector_id, %s, type="eq")' % to_domain.id

    consumer_inst = handle.query_classid(
        class_id='ConsumerInst', filter_str=filter_str)

    if (len(consumer_inst) <= 0):
        raise UcscOperationError("Import config domain",
                                 "Unable to get Domain instance"
                                 "with IP %s for import " %
                                 to_domain.address)

    consumer_dn = consumer_inst[0].dn
    top_system = handle.query_dn("sys")

    mgmt_importer = MgmtImporter(
        parent_mo_or_dn=consumer_dn,
        hostname=top_system.address,
        remote_file='/' + from_domain.address + '/cfg-backups/' + config_file,
        proto=MgmtImporterConsts.PROTO_TFTP,
        admin_state=MgmtImporterConsts.ADMIN_STATE_ENABLED)

    if merge:
        mgmt_importer.action = MgmtImporterConsts.ACTION_MERGE
    else:
        mgmt_importer.action = MgmtImporterConsts.ACTION_REPLACE

    handle.add_mo(mgmt_importer, modify_present=True)
    handle.commit(dme="operation-mgr")

    duration = timeout
    poll_interval = 2
    log.debug("Importing domain config")
    while True:
        mgmt_importer = handle.query_dn(
                dn=mgmt_importer.dn,
                dme="operation-mgr")
        admin_state = mgmt_importer.admin_state

        # Break condition:- if state id disabled then break
        if admin_state == MgmtImporterConsts.ADMIN_STATE_DISABLED:
            break

        time.sleep(min(duration, poll_interval))
        duration = max(0, (duration - poll_interval))
        if duration == 0:
            raise UcscOperationError(
                  "Import config", "operation timed out")

    if mgmt_importer.op_status != \
            MgmtImporterConsts.OP_STATUS_ALL_SUCCESS:
            raise UcscOperationError(
                  "Import config",
                  ("operational status %s " % mgmt_importer.op_status))

    log.debug("Import config to domain was successfull")
    return mgmt_importer


def _schedule_backup(handle, descr, sched_name, max_bkup_files, remote_enabled,
                     protocol, hostname, file_path,
                     username, password, parent_mo_or_dn):

    """
    Internal method to schedule and take remote backup of UcsCentral and Domain
    full-state backup
    """
    from ..mometa.mgmt.MgmtBackupPolicy import MgmtBackupPolicy, \
        MgmtBackupPolicyConsts

    if remote_enabled:
        if not file_path:
            raise UcscValidationException("Missing file_path argument")

        _validate_remote_host_args(protocol, hostname, username, password)
        proto = protocol
        host = hostname
        remote_file = file_path
        user = username
        pwd = password
    else:
        proto = MgmtBackupPolicyConsts.PROTO_NFS_COPY
        host = ""
        remote_file = " "
        user = ""
        pwd = ""

    backup_pol = MgmtBackupPolicy(
            parent_mo_or_dn=parent_mo_or_dn,
            descr=descr,
            admin_state=MgmtBackupPolicyConsts.ADMIN_STATE_ENABLE,
            sched_name=sched_name,
            max_files=str(max_bkup_files),
            proto=proto,
            host=host,
            remote_file=remote_file,
            user=user,
            pwd=pwd,
            name="default")

    handle.add_mo(backup_pol, modify_present=True)
    handle.commit()

    return backup_pol


def schedule_backup(handle, descr="Database Backup Policy",
                    sched_name="global-default", max_bkup_files="2",
                    remote_enabled=False, protocol=None,
                    hostname=None, file_path=None,
                    username=None, password=""):
    """
    schedule_backup helps schedule and optionally take remote backup of
    UcsCentral full-state backup.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        descr (str): Description of the policy
        sched_name (str): Name of the schedule
        max_bkup_files (str): Number of files to keep as backup
        remote_enabled (boolean): True if Remote backup is enabled
                                  False - by default
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        hostname (str): Hostname/IP for the remote backup
        file_path (str): Absolute file path on remote host
        username (str): Username for remote backup
        password (str): Password for remote backup

    Example:
        file_path = "/ws/usr-admin/backup.tgz"\n
        schedule_backup(handle, max_bkup_files=3)

        schedule_backup(handle, file_path=file_path,
                        remote_enabled=True,
                        protocol='scp',hostname='192.168.1.1',
                        username='admin',password='password')\n

    """

    return _schedule_backup(
           handle, descr=descr, sched_name=sched_name,
           max_bkup_files=max_bkup_files,
           remote_enabled=remote_enabled, protocol=protocol,
           hostname=hostname, file_path=file_path,
           username=username, password=password,
           parent_mo_or_dn=ucsc_base_dn)


def schedule_backup_domain(handle, descr="Database Backup Policy",
                           sched_name="global-default", max_bkup_files="2",
                           remote_enabled=False, protocol=None,
                           hostname=None, file_path=None,
                           username=None, password="",
                           domain_group="root"):
    """
    schedule_backup_domain helps schedule and optionally take remote backup of
    domain's full-state backup.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        descr (str): Description of the policy
        sched_name (str): Name of the schedule
        max_bkup_files (str): Number of files to keep as backup
        remote_enabled (boolean): True if Remote backup is enabled
                                  False - by default
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        hostname (str): Hostname/IP for the remote backup
        file_path (str): Absolute file path on remote host
        username (str): Username for remote backup
        password (str): Password for remote backup
        domain_group (str): Fully qualified domain group name

    Example:
        file_path = "/ws/usr-admin/backup.tgz"\n
        schedule_backup_domain(handle, max_bkup_files=3)

        schedule_backup_domain(handle, file_path=file_path,
                               remote_enabled=True,
                               protocol='scp',hostname='192.168.1.1',
                               username='admin',password='password',
                               domain_group="root/demo_domgrp")\n

    """

    from ucscsdk.utils.ucscdomain import get_domain_group_dn
    domain_group_dn = get_domain_group_dn(handle, domain_group)

    return _schedule_backup(
           handle, descr=descr, sched_name=sched_name,
           max_bkup_files=max_bkup_files,
           remote_enabled=remote_enabled, protocol=protocol,
           hostname=hostname, file_path=file_path,
           username=username, password=password,
           parent_mo_or_dn=domain_group_dn)


def _schedule_export_config(handle, descr, sched_name,
                            max_bkup_files, remote_enabled,
                            protocol, hostname, file_path,
                            username, password,
                            parent_mo_or_dn):

    """
    Internal method to schedule and take remote backup of UcsCentral and Domain
    config-all backup
    """
    from ..mometa.mgmt.MgmtCfgExportPolicy import MgmtCfgExportPolicy, \
        MgmtCfgExportPolicyConsts

    if remote_enabled:
        if not file_path:
            raise UcscValidationException("Missing file_path argument")

        _validate_remote_host_args(protocol, hostname, username, password)
        proto = protocol
        host = hostname
        remote_file = file_path
        user = username
        pwd = password
    else:
        proto = MgmtCfgExportPolicyConsts.PROTO_NFS_COPY
        host = ""
        remote_file = " "
        user = ""
        pwd = ""

    cfg_export_pol = MgmtCfgExportPolicy(
            parent_mo_or_dn=parent_mo_or_dn,
            descr=descr,
            admin_state=MgmtCfgExportPolicyConsts.ADMIN_STATE_ENABLE,
            sched_name=sched_name,
            max_files=str(max_bkup_files),
            proto=proto,
            host=host,
            remote_file=remote_file,
            user=user,
            pwd=pwd,
            name="default")

    handle.add_mo(cfg_export_pol, modify_present=True)
    handle.commit()

    return cfg_export_pol


def schedule_export_config(handle, descr="Configuration Export "
                           "Policy", sched_name="global-default",
                           max_bkup_files="2",
                           remote_enabled=False, protocol=None,
                           hostname="", file_path="",
                           username=None, password=""):
    """
    schedule_export_config helps schedule and optionally take remote backup of
    UcsCentral's config-all backup.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        descr (str): Description of the policy
        sched_name (str): Name of the schedule
        max_bkup_files (str): Number of files to keep as backup
        remote_enabled (boolean): True if Remote backup is enabled
                                  False - by default
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        hostname (str): Hostname/IP for the remote backup
        file_path (str): Absolute file path on remote host
        username (str): Username for remote backup
        password (str): Password for remote backup

    Example:
        file_path = "/ws/usr-admin/config.tgz"\n
        schedule_export_config(handle, max_bkup_files=3)

        schedule_export_config(handle, file_path=file_path,
                               remote_enabled=True,
                               protocol='scp',hostname='192.16.1.1',
                               username='admin',password='pass')\n

    """

    return _schedule_export_config(
           handle, descr=descr, sched_name=sched_name,
           max_bkup_files=max_bkup_files,
           remote_enabled=remote_enabled, protocol=protocol,
           hostname=hostname, file_path=file_path,
           username=username, password=password,
           parent_mo_or_dn=ucsc_base_dn)


def schedule_export_config_domain(handle, descr="Configuration Export "
                                  "Policy", sched_name="global-default",
                                  max_bkup_files="2",
                                  remote_enabled=False, protocol=None,
                                  hostname="", file_path="",
                                  username=None, password=None,
                                  domain_group="root"):
    """
    schedule_export_config_domain helps schedule and optionally take remote
    backup of domain's config-all backup.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        descr (str): Description of the policy
        sched_name (str): Name of the schedule
        max_bkup_files (str): Number of files to keep as backup
        remote_enabled (boolean): True if Remote backup is enabled
                                  False - by default
        protocol (str): Transfer protocol for remote backup
                        ['ftp','sftp','tftp','scp']
        hostname (str): Hostname/IP for the remote backup
        file_path (str): Absolute file path on remote host
        username (str): Username for remote backup
        password (str): Password for remote backup
        domain_group (str): Fully qualified domain group name

    Example:
        file_path = "/ws/usr-admin/config.tgz"\n
        schedule_export_config_domain(handle, max_bkup_files=3)

        schedule_export_config_domain(handle, file_path=file_path,
                                      remote_enabled=True,
                                      protocol='scp',hostname='192.16.1.1',
                                      username='admin',password='pass',
                                      domain_group="root/demo_domgrp")\n

    """
    from ucscsdk.utils.ucscdomain import get_domain_group_dn
    domain_group_dn = get_domain_group_dn(handle, domain_group)

    return _schedule_export_config(
           handle, descr=descr, sched_name=sched_name,
           max_bkup_files=max_bkup_files,
           remote_enabled=remote_enabled, protocol=protocol,
           hostname=hostname, file_path=file_path,
           username=username, password=password,
           parent_mo_or_dn=domain_group_dn)


def remove_schedule_backup(handle):
    """
    remvove_schedule_backup removes the schedule of UcsCentral's
    full backup.

    Args:
        handle (UcscHandle): UcsCentral Connection handle

    Example:
        remove_schedule_backup(handle)

    """

    from ..mometa.mgmt.MgmtBackupPolicy import MgmtBackupPolicyConsts
    dn = ucsc_base_dn + "/db-backup-policy-default"

    mo = handle.query_dn(dn=dn)
    if not mo:
        raise UcscOperationError("Remove backup schedule",
                                 "Backup Schedule doesn't exist")

    mo.admin_state = MgmtBackupPolicyConsts.ADMIN_STATE_DISABLE

    handle.set_mo(mo)
    handle.commit()


def remove_schedule_export_config(handle):
    """
    remvove_schedule_export_config removes the schedule of
    UcsCentral config-all backup.

    Args:
        handle (UcscHandle): UcsCentral Connection handle

    Example:
        remove_schedule_export_config(handle)

    """

    from ..mometa.mgmt.MgmtCfgExportPolicy import MgmtCfgExportPolicyConsts
    dn = ucsc_base_dn + "/cfg-exp-policy-default"

    mo = handle.query_dn(dn=dn)
    if not mo:
        raise UcscOperationError("Remove export config",
                                 "Export config schedule doesn't exist")

    mo.admin_state = MgmtCfgExportPolicyConsts.ADMIN_STATE_DISABLE

    handle.set_mo(mo)
    handle.commit()


def remove_schedule_backup_domain(handle, domain_group="root"):
    """
    remvove_schedule_backup_domain removes the schedule policy of Domain's
    full backup.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        domain_group (str): Fully qualified domain group name

    Example:
        remove_schedule_backup_domain(handle, domain_group="root/demo_domgrp")

    """

    from ..mometa.mgmt.MgmtBackupPolicy import MgmtBackupPolicyConsts
    from ucscsdk.utils.ucscdomain import get_domain_group_dn
    domain_group_dn = get_domain_group_dn(handle, domain_group)

    dn = domain_group_dn + "/db-backup-policy-default"

    mo = handle.query_dn(dn=dn)
    if not mo:
        raise UcscOperationError("Remove backup schedule",
                                 "Backup Schedule doesn't exist")

    mo.admin_state = MgmtBackupPolicyConsts.ADMIN_STATE_DISABLE

    handle.set_mo(mo)
    if domain_group != "root":
        handle.remove_mo(mo)
    handle.commit()


def remove_schedule_export_config_domain(handle, domain_group="root"):
    """
    remvove_schedule_export_config_domain removes the schedule policy of
    Domain's config-all backup.

    Args:
        handle (UcscHandle): UcsCentral Connection handle
        domain_group (str): Fully qualified domain group name

    Example:
        remove_schedule_export_config_domain(handle,
                               domain_group="root/demo_domgrp")\n

    """

    from ..mometa.mgmt.MgmtCfgExportPolicy import MgmtCfgExportPolicyConsts
    from ucscsdk.utils.ucscdomain import get_domain_group_dn
    domain_group_dn = get_domain_group_dn(handle, domain_group)

    dn = domain_group_dn + "/cfg-exp-policy-default"

    mo = handle.query_dn(dn=dn)
    if not mo:
        raise UcscOperationError("Remove export config",
                                 "Export config schedule doesn't exist")

    mo.admin_state = MgmtCfgExportPolicyConsts.ADMIN_STATE_DISABLE

    handle.set_mo(mo)
    if domain_group != "root":
        handle.remove_mo(mo)
    handle.commit()



"""This module contains the general information for ConfigNetRefItem ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class ConfigNetRefItemConsts():
    TYPE_ACI = "aci"
    TYPE_ALL = "all"
    TYPE_CLASSIC = "classic"


class ConfigNetRefItem(ManagedObject):
    """This is ConfigNetRefItem class."""

    consts = ConfigNetRefItemConsts()
    naming_props = set([])

    mo_meta = MoMeta("ConfigNetRefItem", "configNetRefItem", "", VersionMeta.Version201b, "InputOutput", 0xf, [], ["read-only"], [], [], [None])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version201b, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, 0x2, 0, 256, None, [], []), 
        "inband_policy_dn": MoPropertyMeta("inband_policy_dn", "inbandPolicyDn", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []), 
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "net_group_dn": MoPropertyMeta("net_group_dn", "netGroupDn", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version201b, MoPropertyMeta.READ_WRITE, 0x8, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
        "type": MoPropertyMeta("type", "type", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["aci", "all", "classic"], []), 
    }

    prop_map = {
        "childAction": "child_action", 
        "dn": "dn", 
        "inbandPolicyDn": "inband_policy_dn", 
        "name": "name", 
        "netGroupDn": "net_group_dn", 
        "rn": "rn", 
        "status": "status", 
        "type": "type", 
    }

    def __init__(self, parent_mo_or_dn, **kwargs):
        self._dirty_mask = 0
        self.child_action = None
        self.inband_policy_dn = None
        self.name = None
        self.net_group_dn = None
        self.status = None
        self.type = None

        ManagedObject.__init__(self, "ConfigNetRefItem", parent_mo_or_dn, **kwargs)




"""This module contains the general information for InitiatorGroupEp ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class InitiatorGroupEpConsts():
    LC_ALLOCATED = "allocated"
    LC_AVAILABLE = "available"
    LC_DEALLOCATED = "deallocated"
    LC_REPURPOSED = "repurposed"


class InitiatorGroupEp(ManagedObject):
    """This is InitiatorGroupEp class."""

    consts = InitiatorGroupEpConsts()
    naming_props = set(['name'])

    mo_meta = MoMeta("InitiatorGroupEp", "initiatorGroupEp", "grp-[name]", VersionMeta.Version131a, "InputOutput", 0x1f, [], ["read-only"], ['initiatorRequestorEp'], ['initiatorFcInitiatorEp', 'initiatorIScsiInitiatorEp', 'initiatorStoreEp', 'storageIScsiInitiatorEp', 'storageNodeEp'], [None])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version131a, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, 0x2, 0, 256, None, [], []), 
        "ep_dn": MoPropertyMeta("ep_dn", "epDn", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []), 
        "id": MoPropertyMeta("id", "id", "uint", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []), 
        "lc": MoPropertyMeta("lc", "lc", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["allocated", "available", "deallocated", "repurposed"], []), 
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version131a, MoPropertyMeta.NAMING, 0x4, None, None, r"""[\-\.:_a-zA-Z0-9]{1,16}""", [], []), 
        "pol_name": MoPropertyMeta("pol_name", "polName", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, r"""[\-\.:_a-zA-Z0-9]{0,16}""", [], []), 
        "rmt_disk_cfg_name": MoPropertyMeta("rmt_disk_cfg_name", "rmtDiskCfgName", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, r"""[\-\.:_a-zA-Z0-9]{0,16}""", [], []), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, 0x8, 0, 256, None, [], []), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x10, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
    }

    prop_map = {
        "childAction": "child_action", 
        "dn": "dn", 
        "epDn": "ep_dn", 
        "id": "id", 
        "lc": "lc", 
        "name": "name", 
        "polName": "pol_name", 
        "rmtDiskCfgName": "rmt_disk_cfg_name", 
        "rn": "rn", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, name, **kwargs):
        self._dirty_mask = 0
        self.name = name
        self.child_action = None
        self.ep_dn = None
        self.id = None
        self.lc = None
        self.pol_name = None
        self.rmt_disk_cfg_name = None
        self.status = None

        ManagedObject.__init__(self, "InitiatorGroupEp", parent_mo_or_dn, **kwargs)




"""This module contains the meta information of IdentGetIdUsage ExternalMethod."""

from ..ucsccoremeta import MethodMeta, MethodPropertyMeta

method_meta = MethodMeta("IdentGetIdUsage", "identGetIdUsage", "Version142b")

prop_meta = {
    "cookie": MethodPropertyMeta("Cookie", "cookie", "Xs:string", "Version142b", "InputOutput", False),
    "in_pool_dn": MethodPropertyMeta("InPoolDn", "inPoolDn", "ReferenceObject", "Version142b", "Input", False),
    "out_number_available": MethodPropertyMeta("OutNumberAvailable", "outNumberAvailable", "Xs:unsignedInt", "Version142b", "Output", False),
    "out_number_used": MethodPropertyMeta("OutNumberUsed", "outNumberUsed", "Xs:unsignedInt", "Version142b", "Output", False),
}

prop_map = {
    "cookie": "cookie",
    "inPoolDn": "in_pool_dn",
    "outNumberAvailable": "out_number_available",
    "outNumberUsed": "out_number_used",
}




"""This module contains the general information for LsbootEmbeddedLocalDiskImagePath ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class LsbootEmbeddedLocalDiskImagePathConsts():
    pass


class LsbootEmbeddedLocalDiskImagePath(ManagedObject):
    """This is LsbootEmbeddedLocalDiskImagePath class."""

    consts = LsbootEmbeddedLocalDiskImagePathConsts()
    naming_props = set(['type'])

    mo_meta = MoMeta("LsbootEmbeddedLocalDiskImagePath", "lsbootEmbeddedLocalDiskImagePath", "diskimgpath-[type]", VersionMeta.Version141a, "InputOutput", 0x3f, [], ["admin", "ls-compute", "ls-config", "ls-config-policy", "ls-server", "ls-server-policy", "ls-storage", "ls-storage-policy"], ['lsbootEmbeddedLocalDiskImage'], ['lsbootUEFIBootParam'], ["Add", "Get", "Remove", "Set"])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version141a, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version141a, MoPropertyMeta.READ_ONLY, 0x2, 0, 256, None, [], []), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version141a, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []), 
        "slot_number": MoPropertyMeta("slot_number", "slotNumber", "ushort", VersionMeta.Version141a, MoPropertyMeta.READ_WRITE, 0x8, None, None, None, [], ["1-254"]), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version141a, MoPropertyMeta.READ_WRITE, 0x10, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
        "type": MoPropertyMeta("type", "type", "string", VersionMeta.Version141a, MoPropertyMeta.NAMING, 0x20, None, None, None, [], []), 
    }

    prop_map = {
        "childAction": "child_action", 
        "dn": "dn", 
        "rn": "rn", 
        "slotNumber": "slot_number", 
        "status": "status", 
        "type": "type", 
    }

    def __init__(self, parent_mo_or_dn, type, **kwargs):
        self._dirty_mask = 0
        self.type = type
        self.child_action = None
        self.slot_number = None
        self.status = None

        ManagedObject.__init__(self, "LsbootEmbeddedLocalDiskImagePath", parent_mo_or_dn, **kwargs)




"""This module contains the general information for FirmwareStatus ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class FirmwareStatusConsts():
    OPER_STATE_ACTIVATING = "activating"
    OPER_STATE_BAD_IMAGE = "bad-image"
    OPER_STATE_FAILED = "failed"
    OPER_STATE_PENDING_NEXT_BOOT = "pending-next-boot"
    OPER_STATE_READY = "ready"
    OPER_STATE_REBOOTING = "rebooting"
    OPER_STATE_SCHEDULED = "scheduled"
    OPER_STATE_SET_STARTUP = "set-startup"
    OPER_STATE_THROTTLED = "throttled"
    OPER_STATE_UPDATING = "updating"
    OPER_STATE_UPGRADING = "upgrading"


class FirmwareStatus(ManagedObject):
    """This is FirmwareStatus class."""

    consts = FirmwareStatusConsts()
    naming_props = set([])

    mo_meta = MoMeta("FirmwareStatus", "firmwareStatus", "fw-status", VersionMeta.Version101a, "InputOutput", 0xf, [], ["admin"], ['computeBlade', 'computeRackUnit', 'computeServerUnit', 'equipmentFex', 'equipmentIOCard', 'equipmentPsu', 'networkElement', 'topSystem'], [], ["Get"])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version101a, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version101a, MoPropertyMeta.READ_ONLY, 0x2, 0, 256, None, [], []), 
        "oper_state": MoPropertyMeta("oper_state", "operState", "string", VersionMeta.Version101a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["activating", "bad-image", "failed", "pending-next-boot", "ready", "rebooting", "scheduled", "set-startup", "throttled", "updating", "upgrading"], []), 
        "package_version": MoPropertyMeta("package_version", "packageVersion", "string", VersionMeta.Version101a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version101a, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []), 
        "service_pack_version": MoPropertyMeta("service_pack_version", "servicePackVersion", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version101a, MoPropertyMeta.READ_WRITE, 0x8, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
    }

    prop_map = {
        "childAction": "child_action", 
        "dn": "dn", 
        "operState": "oper_state", 
        "packageVersion": "package_version", 
        "rn": "rn", 
        "servicePackVersion": "service_pack_version", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, **kwargs):
        self._dirty_mask = 0
        self.child_action = None
        self.oper_state = None
        self.package_version = None
        self.service_pack_version = None
        self.status = None

        ManagedObject.__init__(self, "FirmwareStatus", parent_mo_or_dn, **kwargs)




"""This module contains the general information for LstorageDasScsiLun ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class LstorageDasScsiLunConsts():
    ADMIN_STATE_OFFLINE = "offline"
    ADMIN_STATE_ONLINE = "online"
    ADMIN_STATE_UNDEPLOYED = "undeployed"
    AUTO_DEPLOY_AUTO_DEPLOY = "auto-deploy"
    AUTO_DEPLOY_NO_AUTO_DEPLOY = "no-auto-deploy"
    BOOT_DEV_DISABLED = "disabled"
    BOOT_DEV_ENABLED = "enabled"
    CONFIG_STATE_APPLIED = "applied"
    CONFIG_STATE_APPLYING = "applying"
    CONFIG_STATE_FAILED_TO_APPLY = "failed-to-apply"
    CONFIG_STATE_OK = "ok"
    DEFERRED_NAMING_FALSE = "false"
    DEFERRED_NAMING_NO = "no"
    DEFERRED_NAMING_TRUE = "true"
    DEFERRED_NAMING_YES = "yes"
    EXPAND_TO_AVAIL_FALSE = "false"
    EXPAND_TO_AVAIL_NO = "no"
    EXPAND_TO_AVAIL_TRUE = "true"
    EXPAND_TO_AVAIL_YES = "yes"
    LUN_MAP_TYPE_NON_SHARED = "non-shared"
    LUN_MAP_TYPE_SHARED = "shared"
    LUN_MAP_TYPE_UNASSIGNED = "unassigned"
    OPER_STATE_COMPUTE_DEGRADED = "compute-degraded"
    OPER_STATE_COMPUTE_INOPERABLE = "compute-inoperable"
    OPER_STATE_OFFLINE = "offline"
    OPER_STATE_ONLINE = "online"
    OPER_STATE_UNDEFINED = "undefined"
    ORDER_NOT_APPLICABLE = "not-applicable"
    SIZE_UNSPECIFIED = "unspecified"
    STORAGE_CLASS_DAS = "das"
    STORAGE_CLASS_SAN = "san"


class LstorageDasScsiLun(ManagedObject):
    """This is LstorageDasScsiLun class."""

    consts = LstorageDasScsiLunConsts()
    naming_props = set(['name'])

    mo_meta = MoMeta("LstorageDasScsiLun", "lstorageDasScsiLun", "das-scsi-lun-[name]", VersionMeta.Version131a, "InputOutput", 0x3fff, [], ["admin", "ls-compute", "ls-config", "ls-server", "ls-storage", "ls-storage-policy"], ['lstorageProfile', 'lstorageProfileDef'], ['faultInst', 'storageLocalDiskConfigDef'], ["Add", "Get", "Remove", "Set"])

    prop_meta = {
        "admin_state": MoPropertyMeta("admin_state", "adminState", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x2, None, None, None, ["offline", "online", "undeployed"], []), 
        "auto_deploy": MoPropertyMeta("auto_deploy", "autoDeploy", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x4, None, None, None, ["auto-deploy", "no-auto-deploy"], []), 
        "boot_dev": MoPropertyMeta("boot_dev", "bootDev", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["disabled", "enabled"], []), 
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version131a, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "config_qualifier": MoPropertyMeta("config_qualifier", "configQualifier", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, r"""((defaultValue|not-applicable|identity-assignment|unsupported-storage-capability|lun-id-conflict|missing-firmware-image|lun-capacity-exceeded|insufficient-lun-resources|lun-limit-exceeded|lun-ownership-conflict|storage-unavailable),){0,10}(defaultValue|not-applicable|identity-assignment|unsupported-storage-capability|lun-id-conflict|missing-firmware-image|lun-capacity-exceeded|insufficient-lun-resources|lun-limit-exceeded|lun-ownership-conflict|storage-unavailable){0,1}""", [], []), 
        "config_state": MoPropertyMeta("config_state", "configState", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["applied", "applying", "failed-to-apply", "ok"], []), 
        "deferred_naming": MoPropertyMeta("deferred_naming", "deferredNaming", "string", VersionMeta.Version141a, MoPropertyMeta.READ_WRITE, 0x8, None, None, None, ["false", "no", "true", "yes"], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, 0x10, 0, 256, None, [], []), 
        "expand_to_avail": MoPropertyMeta("expand_to_avail", "expandToAvail", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x20, None, None, None, ["false", "no", "true", "yes"], []), 
        "fractional_size": MoPropertyMeta("fractional_size", "fractionalSize", "ulong", VersionMeta.Version201b, MoPropertyMeta.READ_WRITE, 0x40, None, None, None, [], ["0-1023"]), 
        "local_disk_policy_name": MoPropertyMeta("local_disk_policy_name", "localDiskPolicyName", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x80, None, None, r"""[\-\.:_a-zA-Z0-9]{0,16}""", [], []), 
        "lun_dn": MoPropertyMeta("lun_dn", "lunDn", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []), 
        "lun_map_type": MoPropertyMeta("lun_map_type", "lunMapType", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x100, None, None, None, ["non-shared", "shared", "unassigned"], []), 
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version131a, MoPropertyMeta.NAMING, 0x200, None, None, r"""[\-\.:_a-zA-Z0-9]{1,10}""", [], []), 
        "oper_local_disk_policy_name": MoPropertyMeta("oper_local_disk_policy_name", "operLocalDiskPolicyName", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []), 
        "oper_state": MoPropertyMeta("oper_state", "operState", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["compute-degraded", "compute-inoperable", "offline", "online", "undefined"], []), 
        "order": MoPropertyMeta("order", "order", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x400, None, None, None, ["not-applicable"], ["0-64"]), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, 0x800, 0, 256, None, [], []), 
        "size": MoPropertyMeta("size", "size", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x1000, None, None, None, ["unspecified"], ["0-102400"]), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x2000, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
        "storage_class": MoPropertyMeta("storage_class", "storageClass", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["das", "san"], []), 
    }

    prop_map = {
        "adminState": "admin_state", 
        "autoDeploy": "auto_deploy", 
        "bootDev": "boot_dev", 
        "childAction": "child_action", 
        "configQualifier": "config_qualifier", 
        "configState": "config_state", 
        "deferredNaming": "deferred_naming", 
        "dn": "dn", 
        "expandToAvail": "expand_to_avail", 
        "fractionalSize": "fractional_size", 
        "localDiskPolicyName": "local_disk_policy_name", 
        "lunDn": "lun_dn", 
        "lunMapType": "lun_map_type", 
        "name": "name", 
        "operLocalDiskPolicyName": "oper_local_disk_policy_name", 
        "operState": "oper_state", 
        "order": "order", 
        "rn": "rn", 
        "size": "size", 
        "status": "status", 
        "storageClass": "storage_class", 
    }

    def __init__(self, parent_mo_or_dn, name, **kwargs):
        self._dirty_mask = 0
        self.name = name
        self.admin_state = None
        self.auto_deploy = None
        self.boot_dev = None
        self.child_action = None
        self.config_qualifier = None
        self.config_state = None
        self.deferred_naming = None
        self.expand_to_avail = None
        self.fractional_size = None
        self.local_disk_policy_name = None
        self.lun_dn = None
        self.lun_map_type = None
        self.oper_local_disk_policy_name = None
        self.oper_state = None
        self.order = None
        self.size = None
        self.status = None
        self.storage_class = None

        ManagedObject.__init__(self, "LstorageDasScsiLun", parent_mo_or_dn, **kwargs)




"""This module contains the general information for FaultHolder ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class FaultHolderConsts():
    IS_PINNING_CLEARED_FALSE = "false"
    IS_PINNING_CLEARED_NO = "no"
    IS_PINNING_CLEARED_TRUE = "true"
    IS_PINNING_CLEARED_YES = "yes"


class FaultHolder(ManagedObject):
    """This is FaultHolder class."""

    consts = FaultHolderConsts()
    naming_props = set([])

    mo_meta = MoMeta("FaultHolder", "faultHolder", "fault", VersionMeta.Version101a, "InputOutput", 0x1f, [], ["admin", "fault"], ['topRoot'], ['faultTypedHolder'], ["Get"])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version101a, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version101a, MoPropertyMeta.READ_ONLY, 0x2, 0, 256, None, [], []), 
        "is_pinning_cleared": MoPropertyMeta("is_pinning_cleared", "isPinningCleared", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["false", "no", "true", "yes"], []), 
        "last_pin_time": MoPropertyMeta("last_pin_time", "lastPinTime", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, None, None, r"""([0-9]){4}-([0-9]){2}-([0-9]){2}T([0-9]){2}:([0-9]){2}:([0-9]){2}((\.([0-9]){3})){0,1}""", [], []), 
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version101a, MoPropertyMeta.READ_WRITE, 0x4, None, None, r"""[\-\.:_a-zA-Z0-9]{0,16}""", [], []), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version101a, MoPropertyMeta.READ_ONLY, 0x8, 0, 256, None, [], []), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version101a, MoPropertyMeta.READ_WRITE, 0x10, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
        "total_faults": MoPropertyMeta("total_faults", "totalFaults", "ulong", VersionMeta.Version101a, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []), 
    }

    prop_map = {
        "childAction": "child_action", 
        "dn": "dn", 
        "isPinningCleared": "is_pinning_cleared", 
        "lastPinTime": "last_pin_time", 
        "name": "name", 
        "rn": "rn", 
        "status": "status", 
        "totalFaults": "total_faults", 
    }

    def __init__(self, **kwargs):
        self._dirty_mask = 0
        self.child_action = None
        self.is_pinning_cleared = None
        self.last_pin_time = None
        self.name = None
        self.status = None
        self.total_faults = None

        ManagedObject.__init__(self, "FaultHolder", **kwargs)




"""This module contains the meta information of AaaGetUserLocales ExternalMethod."""

from ..ucsccoremeta import MethodMeta, MethodPropertyMeta

method_meta = MethodMeta("AaaGetUserLocales", "aaaGetUserLocales", "Version142b")

prop_meta = {
    "cookie": MethodPropertyMeta("Cookie", "cookie", "Xs:string", "Version142b", "InputOutput", False),
    "in_is_user_remote": MethodPropertyMeta("InIsUserRemote", "inIsUserRemote", "Xs:string", "Version142b", "Input", False),
    "in_user_name": MethodPropertyMeta("InUserName", "inUserName", "Xs:string", "Version142b", "Input", False),
    "out_user_locales": MethodPropertyMeta("OutUserLocales", "outUserLocales", "Xs:string", "Version142b", "Output", False),
}

prop_map = {
    "cookie": "cookie",
    "inIsUserRemote": "in_is_user_remote",
    "inUserName": "in_user_name",
    "outUserLocales": "out_user_locales",
}




"""This module contains the general information for FlowctrlItem ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class FlowctrlItemConsts():
    PRIO_AUTO = "auto"
    PRIO_ON = "on"
    RCV_OFF = "off"
    RCV_ON = "on"
    SND_OFF = "off"
    SND_ON = "on"


class FlowctrlItem(ManagedObject):
    """This is FlowctrlItem class."""

    consts = FlowctrlItemConsts()
    naming_props = set(['name'])

    mo_meta = MoMeta("FlowctrlItem", "flowctrlItem", "policy-[name]", VersionMeta.Version111a, "InputOutput", 0xff, [], ["admin", "ls-network", "ls-network-policy", "ls-qos-policy"], ['flowctrlDefinition'], [], ["Add", "Get", "Remove", "Set"])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version111a, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, 0x2, 0, 256, None, [], []), 
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version111a, MoPropertyMeta.NAMING, 0x4, None, None, r"""[\-\.:_a-zA-Z0-9]{1,16}""", [], []), 
        "prio": MoPropertyMeta("prio", "prio", "string", VersionMeta.Version111a, MoPropertyMeta.READ_WRITE, 0x8, None, None, None, ["auto", "on"], []), 
        "rcv": MoPropertyMeta("rcv", "rcv", "string", VersionMeta.Version111a, MoPropertyMeta.READ_WRITE, 0x10, None, None, None, ["off", "on"], []), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, 0x20, 0, 256, None, [], []), 
        "snd": MoPropertyMeta("snd", "snd", "string", VersionMeta.Version111a, MoPropertyMeta.READ_WRITE, 0x40, None, None, None, ["off", "on"], []), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version111a, MoPropertyMeta.READ_WRITE, 0x80, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
    }

    prop_map = {
        "childAction": "child_action", 
        "dn": "dn", 
        "name": "name", 
        "prio": "prio", 
        "rcv": "rcv", 
        "rn": "rn", 
        "snd": "snd", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, name, **kwargs):
        self._dirty_mask = 0
        self.name = name
        self.child_action = None
        self.prio = None
        self.rcv = None
        self.snd = None
        self.status = None

        ManagedObject.__init__(self, "FlowctrlItem", parent_mo_or_dn, **kwargs)




# Copyright 2015 Cisco Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
from __future__ import unicode_literals

import sys
import socket
import ssl

from six.moves import urllib as urllib2
from six.moves import http_client as httplib
from six.moves.urllib import request as Request
from six.moves.urllib.error import HTTPError
from six.moves.urllib.request import HTTPRedirectHandler, HTTPSHandler


import logging

log = logging.getLogger('ucscentral')


class SmartRedirectHandler(HTTPRedirectHandler):
    """This class is to handle redirection error."""
    
    def http_error_301(self, req, fp, code, msg, headers):
        """This is to handle redirection error code 301"""
        resp_status = [code, headers.get('Location')]
        return resp_status

    def http_error_302(self, req, fp, code, msg, headers):
        """This is to handle redirection error code 302"""
        resp_status = [code, headers.get('Location')]
        return resp_status


class TLSHandler(HTTPSHandler):
    """Like HTTPSHandler but more specific"""

    def __init__(self):
        HTTPSHandler.__init__(self)

    def https_open(self, req):
        return self.do_open(TLSConnection, req)


class TLSConnection(httplib.HTTPSConnection):
    """Like HTTPSConnection but more specific"""

    def __init__(self, host, **kwargs):
        httplib.HTTPSConnection.__init__(self, host, **kwargs)

    def connect(self):
        """Overrides HTTPSConnection.connect to specify TLS version"""
        # Standard implementation from HTTPSConnection, which is not
        # designed for extension, unfortunately
        if sys.version_info >= (2, 7):
            sock = socket.create_connection((self.host, self.port),
                                            self.timeout, self.source_address)
        elif sys.version_info >= (2, 6):
            sock = socket.create_connection((self.host, self.port),
                                            self.timeout)
        else:
            sock = socket.create_connection((self.host, self.port))

        if getattr(self, '_tunnel_host', None):
            self.sock = sock
            self._tunnel()

        if sys.version_info >= (2, 7, 9):
            # Since python 2.7.9, tls 1.1 and 1.2 are supported via
            # SSLContext
            ssl_context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
            ssl_context.options |= ssl.OP_NO_SSLv2
            ssl_context.options |= ssl.OP_NO_SSLv3
            if getattr(self, "key_file", None) and getattr(self, "cert_file", None):
                ssl_context.load_cert_chain(keyfile=self.key_file,
                                            certfile=self.cert_file)
            self.sock = ssl_context.wrap_socket(sock)
        else:
            # fallback to TLSv1
            self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
                                        ssl_version=ssl.PROTOCOL_TLSv1)


class TLS1Handler(HTTPSHandler):
    """Like HTTPSHandler but more specific"""

    def __init__(self):
        HTTPSHandler.__init__(self)

    def https_open(self, req):
        return self.do_open(TLS1Connection, req)


class TLS1Connection(httplib.HTTPSConnection):
    """Like HTTPSConnection but more specific"""

    def __init__(self, host, **kwargs):
        httplib.HTTPSConnection.__init__(self, host, **kwargs)

    def connect(self):
        """Overrides HTTPSConnection.connect to specify TLS version"""
        # Standard implementation from HTTPSConnection, which is not
        # designed for extension, unfortunately
        if sys.version_info >= (2, 7):
            sock = socket.create_connection((self.host, self.port),
                                            self.timeout, self.source_address)
        elif sys.version_info >= (2, 6):
            sock = socket.create_connection((self.host, self.port),
                                            self.timeout)
        else:
            sock = socket.create_connection((self.host, self.port))

        if getattr(self, '_tunnel_host', None):
            self.sock = sock
            self._tunnel()

        # fallback to TLSv1
        self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
                                    ssl_version=ssl.PROTOCOL_TLSv1)


class UcscDriver(object):
    """
    This class is responsible to create http and https connection using urllib
    library.

    Args:
        proxy (str): The proxy object to be used to connect

    Example:
        driver = UcscDriver(proxy="192.168.1.1:80")
    """

    def __init__(self, proxy=None):
        self.__redirect_uri = None
        self.__proxy = proxy
        self.__headers = {}
        self.__handlers = self.__get_handlers()
    
    def update_handlers(self, tls_proto=None):
        self.__handlers = self.__get_handlers(tls_proto)

    def __get_handlers(self, tls_proto=None):
        """
        Internal method to handle redirection and use TLS protocol.
        """

        # tls_handler implements a fallback mechanism for servers that
        # do not support TLS 1.1/1.2
        tls_handler = (TLSHandler, TLS1Handler)[tls_proto == "tlsv1"]
        handlers = [SmartRedirectHandler, tls_handler]
        if self.__proxy:
            proxy_handler = urllib2.ProxyHandler(
                {'http': self.__proxy, 'https': self.__proxy})
            handlers.append(proxy_handler)
        return handlers

    def add_header(self, header_prop, header_value):
        """
        Adds header to https web request

        Args:
            header_prop (str): header name
            header (str): header value

        Returns:
            None

        Example:
            driver=UcscDriver()\n
            driver.add_header('Cookie', 'xxxxxxxxxxxxxx')\n
        """

        self.__headers[header_prop] = header_value

    def remove_header(self, header_prop):
        """
        Removes header from https web request

        Args:
            header_prop (str): header name

        Returns:
            None

        Example:
            driver=UcscDriver()\n
            driver.remove_header('Cookie')\n
        """

        del self.__headers[header_prop]

    @property
    def redirect_uri(self):
        """
        Getter method of UcsDriver class
        """
        return self.__redirect_uri

    def __create_request(self, uri, data=None):
        """
        Internal method to create http/https web request

        Args:
            uri (str): protocol://web_address:port
            data (str): data to send over web request

        Returns:
            web request object
        """

        request_ = Request.Request(url=uri, data=data)
        headers = self.__headers
        for header in headers:
            request_.add_header(header, headers[header])
        return request_

    def get(self, uri):
        pass

    def post(self, uri, data=None, dump_xml=False, read=True, timeout=None):
        """
        sends the web request and receives the response from UcsCentral server

        Args:
            uri (str): URI of the  the UcsCentral Server
            data (str): request data to send via post request
            dump_xml (bool): if True, displays request and response
            read (bool): if True, returns response.read() else returns object.
            timeout (int): if set, this will be used as timeout in secs for urllib2

        Returns:
            response xml string or response object

        Example:
            response = post("http://192.168.1.1:80", data=xml_str)
        """

        try:
            if self.__redirect_uri:
                uri = self.__redirect_uri
            request = self.__create_request(uri=uri, data=data)
            if dump_xml:
                log.debug('%s ====> %s' % (uri, data))

            opener = Request.build_opener(*self.__handlers)
            try:
                response = opener.open(request, timeout=timeout)
            except Exception as e:
                if "SSL".lower() not in str(e).lower():
                    raise
                # Fallback to TLSv1 for this server
                self.update_handlers(tls_proto="tlsv1")
                opener = Request.build_opener(*self.__handlers)
                response = opener.open(request, timeout=timeout)
            if type(response) is list:
                if len(response) == 2 and \
                        (response[0] == 302 or response[0] == 301):
                    uri = response[1]
                    self.__redirect = True
                    self.__redirect_uri = uri
                    request = self.__create_request(uri=uri, data=data)
                    if dump_xml:
                        log.debug('%s <==== %s' % (uri, data))
                    opener = Request.build_opener(*self.__handlers)
                    response = opener.open(request, timeout=timeout)
                    # response = urllib2.urlopen(request)

            if read:
                response = response.read().decode('utf-8')
                if dump_xml:
                    log.debug('%s <==== %s' % (uri, response))
            return response
        except HTTPError as e:
            log.debug(e.headers)
            raise



"""This module contains the general information for TagDef ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class TagDefConsts():
    MULTIPLE_FALSE = "false"
    MULTIPLE_NO = "no"
    MULTIPLE_TRUE = "true"
    MULTIPLE_YES = "yes"
    RESTRICTED_FALSE = "false"
    RESTRICTED_NO = "no"
    RESTRICTED_TRUE = "true"
    RESTRICTED_YES = "yes"
    SYSTEM_DEFINED_FALSE = "false"
    SYSTEM_DEFINED_NO = "no"
    SYSTEM_DEFINED_TRUE = "true"
    SYSTEM_DEFINED_YES = "yes"


class TagDef(ManagedObject):
    """This is TagDef class."""

    consts = TagDefConsts()
    naming_props = set(['name'])

    mo_meta = MoMeta("TagDef", "tagDef", "type-[name]", VersionMeta.Version151a, "InputOutput", 0x1ff, [], ["admin", "tag"], ['tagDefEp'], ['tagItem'], ["Add", "Get", "Remove", "Set"])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version151a, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "color": MoPropertyMeta("color", "color", "string", VersionMeta.Version151a, MoPropertyMeta.READ_WRITE, 0x2, 0, 510, None, [], []), 
        "descr": MoPropertyMeta("descr", "descr", "string", VersionMeta.Version151a, MoPropertyMeta.READ_WRITE, 0x4, None, None, r"""[ !#$%&\(\)\*\+,\-\./:;\?@\[\]_\{\|\}~a-zA-Z0-9]{0,256}""", [], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, 0x8, 0, 256, None, [], []), 
        "multiple": MoPropertyMeta("multiple", "multiple", "string", VersionMeta.Version151a, MoPropertyMeta.READ_WRITE, 0x10, None, None, None, ["false", "no", "true", "yes"], []), 
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version151a, MoPropertyMeta.NAMING, 0x20, None, None, r"""[ !#$%&\(\)\*\+,\-\.:;=\?@\[\]_\{\|\}~a-zA-Z0-9]{1,32}""", [], []), 
        "restricted": MoPropertyMeta("restricted", "restricted", "string", VersionMeta.Version151a, MoPropertyMeta.READ_WRITE, 0x40, None, None, None, ["false", "no", "true", "yes"], []), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, 0x80, 0, 256, None, [], []), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version151a, MoPropertyMeta.READ_WRITE, 0x100, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
        "system_defined": MoPropertyMeta("system_defined", "systemDefined", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["false", "no", "true", "yes"], []), 
    }

    prop_map = {
        "childAction": "child_action", 
        "color": "color", 
        "descr": "descr", 
        "dn": "dn", 
        "multiple": "multiple", 
        "name": "name", 
        "restricted": "restricted", 
        "rn": "rn", 
        "status": "status", 
        "systemDefined": "system_defined", 
    }

    def __init__(self, parent_mo_or_dn, name, **kwargs):
        self._dirty_mask = 0
        self.name = name
        self.child_action = None
        self.color = None
        self.descr = None
        self.multiple = None
        self.restricted = None
        self.status = None
        self.system_defined = None

        ManagedObject.__init__(self, "TagDef", parent_mo_or_dn, **kwargs)




"""This module contains the general information for AaaProviderRef ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class AaaProviderRefConsts():
    ORDER_LOWEST_AVAILABLE = "lowest-available"


class AaaProviderRef(ManagedObject):
    """This is AaaProviderRef class."""

    consts = AaaProviderRefConsts()
    naming_props = set(['name'])

    mo_meta = MoMeta("AaaProviderRef", "aaaProviderRef", "provider-ref-[name]", VersionMeta.Version101a, "InputOutput", 0x7f, [], ["aaa", "admin", "domain-group-management"], ['aaaProviderGroup'], [], ["Add", "Get", "Remove", "Set"])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version101a, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "descr": MoPropertyMeta("descr", "descr", "string", VersionMeta.Version101a, MoPropertyMeta.READ_WRITE, 0x2, None, None, r"""[ !#$%&\(\)\*\+,\-\./:;\?@\[\]_\{\|\}~a-zA-Z0-9]{0,256}""", [], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version101a, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []), 
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version101a, MoPropertyMeta.NAMING, 0x8, None, None, r"""^[a-zA-Z0-9][a-zA-Z0-9_.-]{0,63}$|^([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}$|^([0-9a-fA-F]{1,4}:){1,7}:$|^([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}$|^([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}$|^([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}$|^([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}$|^([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}$|^[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})$|^:((:[0-9a-fA-F]{1,4}){1,7}|:)$""", [], []), 
        "order": MoPropertyMeta("order", "order", "string", VersionMeta.Version101a, MoPropertyMeta.READ_WRITE, 0x10, None, None, None, ["lowest-available"], ["0-16"]), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version101a, MoPropertyMeta.READ_ONLY, 0x20, 0, 256, None, [], []), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version101a, MoPropertyMeta.READ_WRITE, 0x40, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
    }

    prop_map = {
        "childAction": "child_action", 
        "descr": "descr", 
        "dn": "dn", 
        "name": "name", 
        "order": "order", 
        "rn": "rn", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, name, **kwargs):
        self._dirty_mask = 0
        self.name = name
        self.child_action = None
        self.descr = None
        self.order = None
        self.status = None

        ManagedObject.__init__(self, "AaaProviderRef", parent_mo_or_dn, **kwargs)




"""This module contains the general information for ConfigStorageArrayProfileFilter ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class ConfigStorageArrayProfileFilterConsts():
    ASSIGN_STATE_ASSIGNED = "assigned"
    ASSIGN_STATE_FAILED = "failed"
    ASSIGN_STATE_UNASSIGNED = "unassigned"
    ASSOC_STATE_ASSOCIATED = "associated"
    ASSOC_STATE_ASSOCIATING = "associating"
    ASSOC_STATE_DISASSOCIATING = "disassociating"
    ASSOC_STATE_FAILED = "failed"
    ASSOC_STATE_UNASSOCIATED = "unassociated"
    FAULT_LEVEL_CLEARED = "cleared"
    FAULT_LEVEL_CONDITION = "condition"
    FAULT_LEVEL_CRITICAL = "critical"
    FAULT_LEVEL_INFO = "info"
    FAULT_LEVEL_MAJOR = "major"
    FAULT_LEVEL_MINOR = "minor"
    FAULT_LEVEL_WARNING = "warning"
    OPER_STATE_BIOS_RESTORE = "bios-restore"
    OPER_STATE_CMOS_RESET = "cmos-reset"
    OPER_STATE_COMPUTE_FAILED = "compute-failed"
    OPER_STATE_COMPUTE_MISMATCH = "compute-mismatch"
    OPER_STATE_CONFIG = "config"
    OPER_STATE_CONFIG_FAILURE = "config-failure"
    OPER_STATE_DECOMISSIONING = "decomissioning"
    OPER_STATE_DEGRADED = "degraded"
    OPER_STATE_DIAGNOSTICS = "diagnostics"
    OPER_STATE_DIAGNOSTICS_FAILED = "diagnostics-failed"
    OPER_STATE_DISABLED = "disabled"
    OPER_STATE_DISCOVERY = "discovery"
    OPER_STATE_DISCOVERY_FAILED = "discovery-failed"
    OPER_STATE_INACCESSIBLE = "inaccessible"
    OPER_STATE_INDETERMINATE = "indeterminate"
    OPER_STATE_INOPERABLE = "inoperable"
    OPER_STATE_MAINTENANCE = "maintenance"
    OPER_STATE_MAINTENANCE_FAILED = "maintenance-failed"
    OPER_STATE_OK = "ok"
    OPER_STATE_PENDING_REASSOCIATION = "pending-reassociation"
    OPER_STATE_PENDING_REBOOT = "pending-reboot"
    OPER_STATE_POWER_OFF = "power-off"
    OPER_STATE_POWER_PROBLEM = "power-problem"
    OPER_STATE_REMOVED = "removed"
    OPER_STATE_RESTART = "restart"
    OPER_STATE_TEST = "test"
    OPER_STATE_TEST_FAILED = "test-failed"
    OPER_STATE_THERMAL_PROBLEM = "thermal-problem"
    OPER_STATE_UNASSOCIATED = "unassociated"
    OPER_STATE_UNCONFIG = "unconfig"
    OPER_STATE_UNCONFIG_FAILED = "unconfig-failed"
    OPER_STATE_VOLTAGE_PROBLEM = "voltage-problem"
    OWNER_ALL = "all"
    OWNER_GLOBAL = "global"
    OWNER_LOCAL = "local"
    TYPE_INITIAL_TEMPLATE = "initial-template"
    TYPE_INSTANCE = "instance"
    TYPE_UPDATING_TEMPLATE = "updating-template"


class ConfigStorageArrayProfileFilter(ManagedObject):
    """This is ConfigStorageArrayProfileFilter class."""

    consts = ConfigStorageArrayProfileFilterConsts()
    naming_props = set([])

    mo_meta = MoMeta("ConfigStorageArrayProfileFilter", "configStorageArrayProfileFilter", "storage-array-profile-filter", VersionMeta.Version141a, "InputOutput", 0x3ff, [], ["read-only"], [], [], [None])

    prop_meta = {
        "assign_state": MoPropertyMeta("assign_state", "assignState", "string", VersionMeta.Version141a, MoPropertyMeta.READ_WRITE, 0x2, None, None, None, ["assigned", "failed", "unassigned"], []), 
        "assoc_state": MoPropertyMeta("assoc_state", "assocState", "string", VersionMeta.Version141a, MoPropertyMeta.READ_WRITE, 0x4, None, None, None, ["associated", "associating", "disassociating", "failed", "unassociated"], []), 
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version141a, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version141a, MoPropertyMeta.READ_ONLY, 0x8, 0, 256, None, [], []), 
        "fault_level": MoPropertyMeta("fault_level", "faultLevel", "string", VersionMeta.Version141a, MoPropertyMeta.READ_WRITE, 0x10, None, None, None, ["cleared", "condition", "critical", "info", "major", "minor", "warning"], []), 
        "oper_state": MoPropertyMeta("oper_state", "operState", "string", VersionMeta.Version141a, MoPropertyMeta.READ_WRITE, 0x20, None, None, None, ["bios-restore", "cmos-reset", "compute-failed", "compute-mismatch", "config", "config-failure", "decomissioning", "degraded", "diagnostics", "diagnostics-failed", "disabled", "discovery", "discovery-failed", "inaccessible", "indeterminate", "inoperable", "maintenance", "maintenance-failed", "ok", "pending-reassociation", "pending-reboot", "power-off", "power-problem", "removed", "restart", "test", "test-failed", "thermal-problem", "unassociated", "unconfig", "unconfig-failed", "voltage-problem"], []), 
        "owner": MoPropertyMeta("owner", "owner", "string", VersionMeta.Version141a, MoPropertyMeta.READ_WRITE, 0x40, None, None, None, ["all", "global", "local"], []), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version141a, MoPropertyMeta.READ_ONLY, 0x80, 0, 256, None, [], []), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version141a, MoPropertyMeta.READ_WRITE, 0x100, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
        "type": MoPropertyMeta("type", "type", "string", VersionMeta.Version141a, MoPropertyMeta.READ_WRITE, 0x200, None, None, None, ["initial-template", "instance", "updating-template"], []), 
    }

    prop_map = {
        "assignState": "assign_state", 
        "assocState": "assoc_state", 
        "childAction": "child_action", 
        "dn": "dn", 
        "faultLevel": "fault_level", 
        "operState": "oper_state", 
        "owner": "owner", 
        "rn": "rn", 
        "status": "status", 
        "type": "type", 
    }

    def __init__(self, parent_mo_or_dn, **kwargs):
        self._dirty_mask = 0
        self.assign_state = None
        self.assoc_state = None
        self.child_action = None
        self.fault_level = None
        self.oper_state = None
        self.owner = None
        self.status = None
        self.type = None

        ManagedObject.__init__(self, "ConfigStorageArrayProfileFilter", parent_mo_or_dn, **kwargs)




"""This module contains the general information for StorageController ManagedObject."""

from ...ucscmo import ManagedObject
from ...ucsccoremeta import UcscVersion, MoPropertyMeta, MoMeta
from ...ucscmeta import VersionMeta


class StorageControllerConsts():
    ADMIN_ACTION_CLEAR_BOOT_CONFIGURATION = "clear-boot-configuration"
    ADMIN_ACTION_CLEAR_FOREIGN_CONFIGURATION = "clear-foreign-configuration"
    ADMIN_ACTION_DISABLE_SECURITY = "disable-security"
    ADMIN_ACTION_ENABLE_SECURITY = "enable-security"
    ADMIN_ACTION_IMPORT_FOREIGN_CONFIGURATION = "import-foreign-configuration"
    ADMIN_ACTION_MODIFY_KEY = "modify-key"
    ADMIN_ACTION_SKIP_INITIAL_CONFIG = "skip-initial-config"
    ADMIN_ACTION_UNLOCK_DISK = "unlock-disk"
    ADMIN_ACTION_UNPIN_CACHE_ALL = "unpin-cache-all"
    ADMIN_ACTION_UNSPECIFIED = "unspecified"
    ADMIN_ACTION_TRIGGER_CANCELED = "canceled"
    ADMIN_ACTION_TRIGGER_IDLE = "idle"
    ADMIN_ACTION_TRIGGER_TRIGGERED = "triggered"
    CONFIG_STATE_N_A = "N/A"
    CONFIG_STATE_APPLIED = "applied"
    CONFIG_STATE_APPLY_FAILED = "apply-failed"
    CONFIG_STATE_APPLYING = "applying"
    CONFIG_STATE_NOT_APPLIED = "not-applied"
    CONFIG_STATE_NOT_IN_USE = "not-in-use"
    CONFIG_STATE_ORPHANED = "orphaned"
    CONFIG_STATE_UNKNOWN = "unknown"
    CONTROLLER_STATUS_DEGRADED = "degraded"
    CONTROLLER_STATUS_FAILED = "failed"
    CONTROLLER_STATUS_OPTIMAL = "optimal"
    CONTROLLER_STATUS_UNKNOWN = "unknown"
    CONTROLLER_STATUS_UNRESPONSIVE = "unresponsive"
    ID_COUNT_MIN = "min"
    ID_COUNT_UNKNOWN = "unknown"
    LC_ALLOCATED = "allocated"
    LC_AVAILABLE = "available"
    LC_DEALLOCATED = "deallocated"
    LC_REPURPOSED = "repurposed"
    MODE_AHCI = "AHCI"
    MODE_HBA = "HBA"
    MODE_M2_HWRAID = "M2HWRAID"
    MODE_NVME = "NVME"
    MODE_PHBA = "PHBA"
    MODE_PRAID = "PRAID"
    MODE_RAID = "RAID"
    MODE_SAS4 = "SAS4"
    MODE_SWRAID = "SWRAID"
    MODE_XSDS = "XSDS"
    MODE_UNKNOWN = "unknown"
    ON_BOARD_MEMORY_PRESENT_NO = "no"
    ON_BOARD_MEMORY_PRESENT_UNKNOWN = "unknown"
    ON_BOARD_MEMORY_PRESENT_YES = "yes"
    ON_BOARD_MEMORY_SIZE_UNKNOWN = "unknown"
    OOB_CONTROLLER_ID_NOT_APPLICABLE = "not-applicable"
    OOB_INTERFACE_SUPPORTED_FALSE = "false"
    OOB_INTERFACE_SUPPORTED_NO = "no"
    OOB_INTERFACE_SUPPORTED_TRUE = "true"
    OOB_INTERFACE_SUPPORTED_YES = "yes"
    OPER_STATE_ACCESSIBILITY_PROBLEM = "accessibility-problem"
    OPER_STATE_AUTO_UPGRADE = "auto-upgrade"
    OPER_STATE_BACKPLANE_PORT_PROBLEM = "backplane-port-problem"
    OPER_STATE_BIOS_POST_TIMEOUT = "bios-post-timeout"
    OPER_STATE_CHASSIS_LIMIT_EXCEEDED = "chassis-limit-exceeded"
    OPER_STATE_CONFIG = "config"
    OPER_STATE_DECOMISSIONING = "decomissioning"
    OPER_STATE_DEGRADED = "degraded"
    OPER_STATE_DISABLED = "disabled"
    OPER_STATE_DISCOVERY = "discovery"
    OPER_STATE_DISCOVERY_FAILED = "discovery-failed"
    OPER_STATE_EQUIPMENT_PROBLEM = "equipment-problem"
    OPER_STATE_FABRIC_CONN_PROBLEM = "fabric-conn-problem"
    OPER_STATE_FABRIC_UNSUPPORTED_CONN = "fabric-unsupported-conn"
    OPER_STATE_IDENTIFY = "identify"
    OPER_STATE_IDENTITY_UNESTABLISHABLE = "identity-unestablishable"
    OPER_STATE_INOPERABLE = "inoperable"
    OPER_STATE_MALFORMED_FRU = "malformed-fru"
    OPER_STATE_NOT_SUPPORTED = "not-supported"
    OPER_STATE_OPERABLE = "operable"
    OPER_STATE_PEER_COMM_PROBLEM = "peer-comm-problem"
    OPER_STATE_PERFORMANCE_PROBLEM = "performance-problem"
    OPER_STATE_POST_FAILURE = "post-failure"
    OPER_STATE_POWER_PROBLEM = "power-problem"
    OPER_STATE_POWERED_OFF = "powered-off"
    OPER_STATE_REMOVED = "removed"
    OPER_STATE_THERMAL_PROBLEM = "thermal-problem"
    OPER_STATE_UNKNOWN = "unknown"
    OPER_STATE_UPGRADE_PROBLEM = "upgrade-problem"
    OPER_STATE_VOLTAGE_PROBLEM = "voltage-problem"
    OPERABILITY_ACCESSIBILITY_PROBLEM = "accessibility-problem"
    OPERABILITY_AUTO_UPGRADE = "auto-upgrade"
    OPERABILITY_BACKPLANE_PORT_PROBLEM = "backplane-port-problem"
    OPERABILITY_BIOS_POST_TIMEOUT = "bios-post-timeout"
    OPERABILITY_CHASSIS_LIMIT_EXCEEDED = "chassis-limit-exceeded"
    OPERABILITY_CONFIG = "config"
    OPERABILITY_DECOMISSIONING = "decomissioning"
    OPERABILITY_DEGRADED = "degraded"
    OPERABILITY_DISABLED = "disabled"
    OPERABILITY_DISCOVERY = "discovery"
    OPERABILITY_DISCOVERY_FAILED = "discovery-failed"
    OPERABILITY_EQUIPMENT_PROBLEM = "equipment-problem"
    OPERABILITY_FABRIC_CONN_PROBLEM = "fabric-conn-problem"
    OPERABILITY_FABRIC_UNSUPPORTED_CONN = "fabric-unsupported-conn"
    OPERABILITY_IDENTIFY = "identify"
    OPERABILITY_IDENTITY_UNESTABLISHABLE = "identity-unestablishable"
    OPERABILITY_INOPERABLE = "inoperable"
    OPERABILITY_MALFORMED_FRU = "malformed-fru"
    OPERABILITY_NOT_SUPPORTED = "not-supported"
    OPERABILITY_OPERABLE = "operable"
    OPERABILITY_PEER_COMM_PROBLEM = "peer-comm-problem"
    OPERABILITY_PERFORMANCE_PROBLEM = "performance-problem"
    OPERABILITY_POST_FAILURE = "post-failure"
    OPERABILITY_POWER_PROBLEM = "power-problem"
    OPERABILITY_POWERED_OFF = "powered-off"
    OPERABILITY_REMOVED = "removed"
    OPERABILITY_THERMAL_PROBLEM = "thermal-problem"
    OPERABILITY_UNKNOWN = "unknown"
    OPERABILITY_UPGRADE_PROBLEM = "upgrade-problem"
    OPERABILITY_VOLTAGE_PROBLEM = "voltage-problem"
    OPROM_BOOT_STATUS_DISABLED = "disabled"
    OPROM_BOOT_STATUS_ENABLED = "enabled"
    OPROM_BOOT_STATUS_UNKNOWN = "unknown"
    PERF_LOWER_CRITICAL = "lower-critical"
    PERF_LOWER_NON_CRITICAL = "lower-non-critical"
    PERF_LOWER_NON_RECOVERABLE = "lower-non-recoverable"
    PERF_NOT_SUPPORTED = "not-supported"
    PERF_OK = "ok"
    PERF_UNKNOWN = "unknown"
    PERF_UPPER_CRITICAL = "upper-critical"
    PERF_UPPER_NON_CRITICAL = "upper-non-critical"
    PERF_UPPER_NON_RECOVERABLE = "upper-non-recoverable"
    PINNED_CACHE_STATUS_DISABLED = "disabled"
    PINNED_CACHE_STATUS_ENABLED = "enabled"
    PINNED_CACHE_STATUS_UNKNOWN = "unknown"
    POWER_DEGRADED = "degraded"
    POWER_ERROR = "error"
    POWER_FAILED = "failed"
    POWER_NOT_SUPPORTED = "not-supported"
    POWER_OFF = "off"
    POWER_OFFDUTY = "offduty"
    POWER_OFFLINE = "offline"
    POWER_OK = "ok"
    POWER_ON = "on"
    POWER_ONLINE = "online"
    POWER_POWER_SAVE = "power-save"
    POWER_TEST = "test"
    POWER_UNKNOWN = "unknown"
    PRESENCE_EMPTY = "empty"
    PRESENCE_EQUIPPED = "equipped"
    PRESENCE_EQUIPPED_IDENTITY_UNESTABLISHABLE = "equipped-identity-unestablishable"
    PRESENCE_EQUIPPED_NOT_PRIMARY = "equipped-not-primary"
    PRESENCE_EQUIPPED_SLAVE = "equipped-slave"
    PRESENCE_EQUIPPED_UNSUPPORTED = "equipped-unsupported"
    PRESENCE_EQUIPPED_WITH_MALFORMED_FRU = "equipped-with-malformed-fru"
    PRESENCE_INACCESSIBLE = "inaccessible"
    PRESENCE_MISMATCH = "mismatch"
    PRESENCE_MISMATCH_IDENTITY_UNESTABLISHABLE = "mismatch-identity-unestablishable"
    PRESENCE_MISMATCH_SLAVE = "mismatch-slave"
    PRESENCE_MISSING = "missing"
    PRESENCE_MISSING_SLAVE = "missing-slave"
    PRESENCE_NOT_SUPPORTED = "not-supported"
    PRESENCE_UNAUTHORIZED = "unauthorized"
    PRESENCE_UNKNOWN = "unknown"
    REBUILD_RATE_NOT_APPLICABLE = "not-applicable"
    REBUILD_RATE_UNKNOWN = "unknown"
    SUB_OEM_ID_UNKNOWN = "unknown"
    SUB_TYPE_NA = "NA"
    SUB_TYPE_NVME_FRONT = "NVME-FRONT"
    SUB_TYPE_NVME_HHHL = "NVME-HHHL"
    SUB_TYPE_NVME_M2 = "NVME-M2"
    SUB_TYPE_NVME_MEZZ = "NVME-MEZZ"
    SUB_TYPE_NVME_REAR = "NVME-REAR"
    SUB_TYPE_RDE = "RDE"
    THERMAL_LOWER_CRITICAL = "lower-critical"
    THERMAL_LOWER_NON_CRITICAL = "lower-non-critical"
    THERMAL_LOWER_NON_RECOVERABLE = "lower-non-recoverable"
    THERMAL_NOT_SUPPORTED = "not-supported"
    THERMAL_OK = "ok"
    THERMAL_UNKNOWN = "unknown"
    THERMAL_UPPER_CRITICAL = "upper-critical"
    THERMAL_UPPER_NON_CRITICAL = "upper-non-critical"
    THERMAL_UPPER_NON_RECOVERABLE = "upper-non-recoverable"
    TYPE_FLASH = "FLASH"
    TYPE_HBA = "HBA"
    TYPE_NVME = "NVME"
    TYPE_PCH = "PCH"
    TYPE_PT = "PT"
    TYPE_SAS = "SAS"
    TYPE_SATA = "SATA"
    TYPE_SD = "SD"
    TYPE_EXTERNAL = "external"
    TYPE_UNKNOWN = "unknown"
    VOLTAGE_LOWER_CRITICAL = "lower-critical"
    VOLTAGE_LOWER_NON_CRITICAL = "lower-non-critical"
    VOLTAGE_LOWER_NON_RECOVERABLE = "lower-non-recoverable"
    VOLTAGE_NOT_SUPPORTED = "not-supported"
    VOLTAGE_OK = "ok"
    VOLTAGE_UNKNOWN = "unknown"
    VOLTAGE_UPPER_CRITICAL = "upper-critical"
    VOLTAGE_UPPER_NON_CRITICAL = "upper-non-critical"
    VOLTAGE_UPPER_NON_RECOVERABLE = "upper-non-recoverable"


class StorageController(ManagedObject):
    """This is StorageController class."""

    consts = StorageControllerConsts()
    naming_props = set(['type', 'id'])

    mo_meta = MoMeta("StorageController", "storageController", "storage-[type]-[id]", VersionMeta.Version111a, "InputOutput", 0x1ff, [], ["admin", "ls-compute", "ls-config", "ls-server", "ls-storage"], ['computeBoard', 'equipmentChassis'], ['faultInst', 'firmwareBootDefinition', 'firmwareRunning', 'lstorageControllerDef', 'mgmtController', 'storageControllerOperation', 'storageEmbeddedStorage', 'storageEnclosure', 'storageLocalDisk', 'storageLocalDiskConfigDef', 'storageLocalDiskEp', 'storageLocalLun', 'storageMezzFlashLife', 'storageNvmeStats', 'storageNvmeStorage', 'storageOnboardDevice', 'storageOperation', 'storageRaidBattery', 'storageSsdHealthStats', 'storageVirtualDrive', 'storageVirtualDriveEp'], ["Get"])

    prop_meta = {
        "admin_action": MoPropertyMeta("admin_action", "adminAction", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x2, None, None, None, ["clear-boot-configuration", "clear-foreign-configuration", "disable-security", "enable-security", "import-foreign-configuration", "modify-key", "skip-initial-config", "unlock-disk", "unpin-cache-all", "unspecified"], []), 
        "admin_action_trigger": MoPropertyMeta("admin_action_trigger", "adminActionTrigger", "string", VersionMeta.Version131a, MoPropertyMeta.READ_WRITE, 0x4, None, None, None, ["canceled", "idle", "triggered"], []), 
        "admin_security_key": MoPropertyMeta("admin_security_key", "adminSecurityKey", "string", VersionMeta.Version201b, MoPropertyMeta.READ_WRITE, 0x8, 0, 510, None, [], []), 
        "asset_tag": MoPropertyMeta("asset_tag", "assetTag", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, None, None, r"""[ !#$%&\(\)\*\+,\-\./:;\?@\[\]_\{\|\}~a-zA-Z0-9]{0,32}""", [], []), 
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version111a, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []), 
        "config_state": MoPropertyMeta("config_state", "configState", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["N/A", "applied", "apply-failed", "applying", "not-applied", "not-in-use", "orphaned", "unknown"], []), 
        "controller_flags": MoPropertyMeta("controller_flags", "controllerFlags", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, None, None, r"""((defaultValue|none|driveSecurityCapable|driveSecurityEnable),){0,3}(defaultValue|none|driveSecurityCapable|driveSecurityEnable){0,1}""", [], []), 
        "controller_ops": MoPropertyMeta("controller_ops", "controllerOps", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, r"""((defaultValue|unknown|unlock-disk|get-phy-errors|ctlr-shutdown|set-suboem|get-foreign-configuration|no-ops-supported|disable-security|clear-all-config|get-pinned-list|set-factory-defaults|clear-foreign-configuration|modify|update-boot-drive|unpin-cache|get-time-secs|import-foreign-configuration|clear-boot-drive|ctlr-lock-operation|set-time-secs|get-suboem|get-tty-log),){0,22}(defaultValue|unknown|unlock-disk|get-phy-errors|ctlr-shutdown|set-suboem|get-foreign-configuration|no-ops-supported|disable-security|clear-all-config|get-pinned-list|set-factory-defaults|clear-foreign-configuration|modify|update-boot-drive|unpin-cache|get-time-secs|import-foreign-configuration|clear-boot-drive|ctlr-lock-operation|set-time-secs|get-suboem|get-tty-log){0,1}""", [], []), 
        "controller_status": MoPropertyMeta("controller_status", "controllerStatus", "string", VersionMeta.Version112a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["degraded", "failed", "optimal", "unknown", "unresponsive"], []), 
        "default_strip_size": MoPropertyMeta("default_strip_size", "defaultStripSize", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, r"""((defaultValue|unknown|not-applicable|8KB|16KB|32KB|64KB|128KB|256KB|512KB|1MB|2MB|4MB|8MB|16MB|32MB|64MB|128MB|256MB|512MB),){0,19}(defaultValue|unknown|not-applicable|8KB|16KB|32KB|64KB|128KB|256KB|512KB|1MB|2MB|4MB|8MB|16MB|32MB|64MB|128MB|256MB|512MB){0,1}""", [], []), 
        "device_raid_support": MoPropertyMeta("device_raid_support", "deviceRaidSupport", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "disk_ops": MoPropertyMeta("disk_ops", "diskOps", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, r"""((defaultValue|unknown|clear-secure-foreign-config-drive|set-rebuild-status|clear-foreign-configuration|cancel-rebuild|make-offline|make-ghsp|no-ops-supported|get-phy-errors|set-copyback-status|set-state|get-pd-progress|prepare-to-remove|make-dhsp|remove|smart-ssd-support|enable-security-on-jbod|set-boot-drive|get-foreign-configuration|undo-prepare-to-remove|remove-hsp|locate-start|clear-secure-drive|update-lrop-status|import-foreign-configuration|start-rebuild|make-online|locate-stop),){0,28}(defaultValue|unknown|clear-secure-foreign-config-drive|set-rebuild-status|clear-foreign-configuration|cancel-rebuild|make-offline|make-ghsp|no-ops-supported|get-phy-errors|set-copyback-status|set-state|get-pd-progress|prepare-to-remove|make-dhsp|remove|smart-ssd-support|enable-security-on-jbod|set-boot-drive|get-foreign-configuration|undo-prepare-to-remove|remove-hsp|locate-start|clear-secure-drive|update-lrop-status|import-foreign-configuration|start-rebuild|make-online|locate-stop){0,1}""", [], []), 
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, 0x10, 0, 256, None, [], []), 
        "err_description": MoPropertyMeta("err_description", "errDescription", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, 0, 64, None, [], []), 
        "fault_monitoring": MoPropertyMeta("fault_monitoring", "faultMonitoring", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []), 
        "hw_revision": MoPropertyMeta("hw_revision", "hwRevision", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "id": MoPropertyMeta("id", "id", "uint", VersionMeta.Version111a, MoPropertyMeta.NAMING, 0x20, None, None, None, [], ["1-64"]), 
        "id_count": MoPropertyMeta("id_count", "idCount", "string", VersionMeta.Version141a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["min", "unknown"], ["0-4294967295"]), 
        "lc": MoPropertyMeta("lc", "lc", "string", VersionMeta.Version112a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["allocated", "available", "deallocated", "repurposed"], []), 
        "location_dn": MoPropertyMeta("location_dn", "locationDn", "string", VersionMeta.Version121a, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []), 
        "mode": MoPropertyMeta("mode", "mode", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["AHCI", "HBA", "M2HWRAID", "NVME", "PHBA", "PRAID", "RAID", "SAS4", "SWRAID", "XSDS", "unknown"], []), 
        "model": MoPropertyMeta("model", "model", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "on_board_memory_present": MoPropertyMeta("on_board_memory_present", "onBoardMemoryPresent", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["no", "unknown", "yes"], []), 
        "on_board_memory_size": MoPropertyMeta("on_board_memory_size", "onBoardMemorySize", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["unknown"], ["0-4294967295"]), 
        "oob_controller_id": MoPropertyMeta("oob_controller_id", "oobControllerId", "string", VersionMeta.Version112a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["not-applicable"], ["0-4294967295"]), 
        "oob_interface_supported": MoPropertyMeta("oob_interface_supported", "oobInterfaceSupported", "string", VersionMeta.Version112a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["false", "no", "true", "yes"], []), 
        "oper_qualifier_reason": MoPropertyMeta("oper_qualifier_reason", "operQualifierReason", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, None, None, r"""[ !#$%&\(\)\*\+,\-\./:;\?@\[\]_\{\|\}~a-zA-Z0-9]{0,256}""", [], []), 
        "oper_state": MoPropertyMeta("oper_state", "operState", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["accessibility-problem", "auto-upgrade", "backplane-port-problem", "bios-post-timeout", "chassis-limit-exceeded", "config", "decomissioning", "degraded", "disabled", "discovery", "discovery-failed", "equipment-problem", "fabric-conn-problem", "fabric-unsupported-conn", "identify", "identity-unestablishable", "inoperable", "malformed-fru", "not-supported", "operable", "peer-comm-problem", "performance-problem", "post-failure", "power-problem", "powered-off", "removed", "thermal-problem", "unknown", "upgrade-problem", "voltage-problem"], []), 
        "operability": MoPropertyMeta("operability", "operability", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["accessibility-problem", "auto-upgrade", "backplane-port-problem", "bios-post-timeout", "chassis-limit-exceeded", "config", "decomissioning", "degraded", "disabled", "discovery", "discovery-failed", "equipment-problem", "fabric-conn-problem", "fabric-unsupported-conn", "identify", "identity-unestablishable", "inoperable", "malformed-fru", "not-supported", "operable", "peer-comm-problem", "performance-problem", "post-failure", "power-problem", "powered-off", "removed", "thermal-problem", "unknown", "upgrade-problem", "voltage-problem"], []), 
        "oprom_boot_status": MoPropertyMeta("oprom_boot_status", "opromBootStatus", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["disabled", "enabled", "unknown"], []), 
        "part_number": MoPropertyMeta("part_number", "partNumber", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "pci_addr": MoPropertyMeta("pci_addr", "pciAddr", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "pci_slot": MoPropertyMeta("pci_slot", "pciSlot", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "pci_slot_raw_name": MoPropertyMeta("pci_slot_raw_name", "pciSlotRawName", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "perf": MoPropertyMeta("perf", "perf", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["lower-critical", "lower-non-critical", "lower-non-recoverable", "not-supported", "ok", "unknown", "upper-critical", "upper-non-critical", "upper-non-recoverable"], []), 
        "pinned_cache_status": MoPropertyMeta("pinned_cache_status", "pinnedCacheStatus", "string", VersionMeta.Version141a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["disabled", "enabled", "unknown"], []), 
        "power": MoPropertyMeta("power", "power", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["degraded", "error", "failed", "not-supported", "off", "offduty", "offline", "ok", "on", "online", "power-save", "test", "unknown"], []), 
        "presence": MoPropertyMeta("presence", "presence", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["empty", "equipped", "equipped-identity-unestablishable", "equipped-not-primary", "equipped-slave", "equipped-unsupported", "equipped-with-malformed-fru", "inaccessible", "mismatch", "mismatch-identity-unestablishable", "mismatch-slave", "missing", "missing-slave", "not-supported", "unauthorized", "unknown"], []), 
        "raid_battery_ops": MoPropertyMeta("raid_battery_ops", "raidBatteryOps", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, r"""((defaultValue|unknown|no-ops-supported|disable-learn|enable-learn|start-learn-cycle),){0,5}(defaultValue|unknown|no-ops-supported|disable-learn|enable-learn|start-learn-cycle){0,1}""", [], []), 
        "raid_support": MoPropertyMeta("raid_support", "raidSupport", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "rebuild_rate": MoPropertyMeta("rebuild_rate", "rebuildRate", "string", VersionMeta.Version112a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["not-applicable", "unknown"], ["0-101"]), 
        "revision": MoPropertyMeta("revision", "revision", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, 0x40, 0, 256, None, [], []), 
        "serial": MoPropertyMeta("serial", "serial", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version111a, MoPropertyMeta.READ_WRITE, 0x80, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []), 
        "sub_oem_id": MoPropertyMeta("sub_oem_id", "subOemId", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["unknown"], ["0-4294967295"]), 
        "sub_type": MoPropertyMeta("sub_type", "subType", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA", "NVME-FRONT", "NVME-HHHL", "NVME-M2", "NVME-MEZZ", "NVME-REAR", "RDE"], []), 
        "supported_strip_sizes": MoPropertyMeta("supported_strip_sizes", "supportedStripSizes", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, r"""((defaultValue|unknown|not-applicable|8KB|16KB|32KB|64KB|128KB|256KB|512KB|1MB|2MB|4MB|8MB|16MB|32MB|64MB|128MB|256MB|512MB),){0,19}(defaultValue|unknown|not-applicable|8KB|16KB|32KB|64KB|128KB|256KB|512KB|1MB|2MB|4MB|8MB|16MB|32MB|64MB|128MB|256MB|512MB){0,1}""", [], []), 
        "thermal": MoPropertyMeta("thermal", "thermal", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["lower-critical", "lower-non-critical", "lower-non-recoverable", "not-supported", "ok", "unknown", "upper-critical", "upper-non-critical", "upper-non-recoverable"], []), 
        "type": MoPropertyMeta("type", "type", "string", VersionMeta.Version111a, MoPropertyMeta.NAMING, 0x100, None, None, None, ["FLASH", "HBA", "NVME", "PCH", "PT", "SAS", "SATA", "SD", "external", "unknown"], []), 
        "variant_type": MoPropertyMeta("variant_type", "variantType", "string", VersionMeta.Version201b, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "vendor": MoPropertyMeta("vendor", "vendor", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "vid": MoPropertyMeta("vid", "vid", "string", VersionMeta.Version131a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []), 
        "virtual_drive_ops": MoPropertyMeta("virtual_drive_ops", "virtualDriveOps", "string", VersionMeta.Version151a, MoPropertyMeta.READ_ONLY, None, None, None, r"""((defaultValue|unknown|set-changed|clear-transport-ready|set-cc-status|get-maxsize|get-reconstruction-info|cancel-initialization|delete-operation|no-ops-supported|get-disk-group|unpin-cache-op|update-lrop-status|get-ld-progress|start-check-consistency|modify-operation|deleted|set-reconstruction-status|secure-drive-group|set-hidden-op|start-patrol-read|cancel-check-consistency|set-boot-drive-operation|create-operation|set-initialization-status|set-transport-ready|clear-hidden-op|stop-patrol-read|start-reconstruction|start-initialization|carve-operation),){0,30}(defaultValue|unknown|set-changed|clear-transport-ready|set-cc-status|get-maxsize|get-reconstruction-info|cancel-initialization|delete-operation|no-ops-supported|get-disk-group|unpin-cache-op|update-lrop-status|get-ld-progress|start-check-consistency|modify-operation|deleted|set-reconstruction-status|secure-drive-group|set-hidden-op|start-patrol-read|cancel-check-consistency|set-boot-drive-operation|create-operation|set-initialization-status|set-transport-ready|clear-hidden-op|stop-patrol-read|start-reconstruction|start-initialization|carve-operation){0,1}""", [], []), 
        "voltage": MoPropertyMeta("voltage", "voltage", "string", VersionMeta.Version111a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["lower-critical", "lower-non-critical", "lower-non-recoverable", "not-supported", "ok", "unknown", "upper-critical", "upper-non-critical", "upper-non-recoverable"], []), 
    }

    prop_map = {
        "adminAction": "admin_action", 
        "adminActionTrigger": "admin_action_trigger", 
        "adminSecurityKey": "admin_security_key", 
        "assetTag": "asset_tag", 
        "childAction": "child_action", 
        "configState": "config_state", 
        "controllerFlags": "controller_flags", 
        "controllerOps": "controller_ops", 
        "controllerStatus": "controller_status", 
        "defaultStripSize": "default_strip_size", 
        "deviceRaidSupport": "device_raid_support", 
        "diskOps": "disk_ops", 
        "dn": "dn", 
        "errDescription": "err_description", 
        "faultMonitoring": "fault_monitoring", 
        "hwRevision": "hw_revision", 
        "id": "id", 
        "idCount": "id_count", 
        "lc": "lc", 
        "locationDn": "location_dn", 
        "mode": "mode", 
        "model": "model", 
        "onBoardMemoryPresent": "on_board_memory_present", 
        "onBoardMemorySize": "on_board_memory_size", 
        "oobControllerId": "oob_controller_id", 
        "oobInterfaceSupported": "oob_interface_supported", 
        "operQualifierReason": "oper_qualifier_reason", 
        "operState": "oper_state", 
        "operability": "operability", 
        "opromBootStatus": "oprom_boot_status", 
        "partNumber": "part_number", 
        "pciAddr": "pci_addr", 
        "pciSlot": "pci_slot", 
        "pciSlotRawName": "pci_slot_raw_name", 
        "perf": "perf", 
        "pinnedCacheStatus": "pinned_cache_status", 
        "power": "power", 
        "presence": "presence", 
        "raidBatteryOps": "raid_battery_ops", 
        "raidSupport": "raid_support", 
        "rebuildRate": "rebuild_rate", 
        "revision": "revision", 
        "rn": "rn", 
        "serial": "serial", 
        "status": "status", 
        "subOemId": "sub_oem_id", 
        "subType": "sub_type", 
        "supportedStripSizes": "supported_strip_sizes", 
        "thermal": "thermal", 
        "type": "type", 
        "variantType": "variant_type", 
        "vendor": "vendor", 
        "vid": "vid", 
        "virtualDriveOps": "virtual_drive_ops", 
        "voltage": "voltage", 
    }

    def __init__(self, parent_mo_or_dn, type, id, **kwargs):
        self._dirty_mask = 0
        self.type = type
        self.id = id
        self.admin_action = None
        self.admin_action_trigger = None
        self.admin_security_key = None
        self.asset_tag = None
        self.child_action = None
        self.config_state = None
        self.controller_flags = None
        self.controller_ops = None
        self.controller_status = None
        self.default_strip_size = None
        self.device_raid_support = None
        self.disk_ops = None
        self.err_description = None
        self.fault_monitoring = None
        self.hw_revision = None
        self.id_count = None
        self.lc = None
        self.location_dn = None
        self.mode = None
        self.model = None
        self.on_board_memory_present = None
        self.on_board_memory_size = None
        self.oob_controller_id = None
        self.oob_interface_supported = None
        self.oper_qualifier_reason = None
        self.oper_state = None
        self.operability = None
        self.oprom_boot_status = None
        self.part_number = None
        self.pci_addr = None
        self.pci_slot = None
        self.pci_slot_raw_name = None
        self.perf = None
        self.pinned_cache_status = None
        self.power = None
        self.presence = None
        self.raid_battery_ops = None
        self.raid_support = None
        self.rebuild_rate = None
        self.revision = None
        self.serial = None
        self.status = None
        self.sub_oem_id = None
        self.sub_type = None
        self.supported_strip_sizes = None
        self.thermal = None
        self.variant_type = None
        self.vendor = None
        self.vid = None
        self.virtual_drive_ops = None
        self.voltage = None

        ManagedObject.__init__(self, "StorageController", parent_mo_or_dn, **kwargs)

